unit ExtStrm;

interface

uses
  SysUtils, Types, Classes, Variants, QTypes, QGraphics, Libc, Math;
//	Windows, SysUtils, Classes, Math, WinExt{, ExtUtils};

type
	TCustomLightMemoryStream=class(TCustomMemoryStream)
	protected
		constructor	Create(Mem:Pointer;Size:Integer);	overload;	virtual;
		constructor	Create(M:TCustomMemoryStream);	overload;	virtual;
		function	GetAvailable:	Boolean;	virtual;
		function	GetAvailSize:	Integer;	virtual;
		property	Available:	Boolean	read	GetAvailable;
		property	AvailSize:	Integer	read	GetAvailSize;
	public
		function	Write(const	Buffer;Count:Integer):	Longint;	override;
		function	Getc:	Byte;
		procedure	Putc(c:Byte);
		function	Getw:	Word;
		procedure	Putw(w:Word);
		function	Getdw:	DWord;
		procedure	Putdw(dw:DWord);
		procedure	Clear(Value:Char=#0);
	end;

type
	TLightMemoryStream=class(TCustomLightMemoryStream)
	public
		constructor	Create(Mem:Pointer;Size:Integer);	overload;	override;
		constructor	Create(M:TCustomMemoryStream);	overload;	override;
		property	Available;
		property	AvailSize;
	end;

{$IFDEF WINDOWS}
type
	TMemoryHandleStream=class(TCustomLightMemoryStream)
	private
		FHandle:	HLOCAL;
	protected
		property	Handle:	HLOCAL	read	FHandle;
	public
		constructor	Create;	overload;
//		constructor	Create(Size:Longint);	overload;
		constructor	Create(Handle:HLOCAL);	overload;
		destructor	Destroy;	override;
		procedure	Clear(Value:Char=#0);
		procedure	SetSize(NewSize:Longint);	override;
		function	Release:	HLOCAL;
	end;
{$ENDIF}

const
	INFINITE=DWord(-1);

type
	TShareableMemoryStream=class(TCustomLightMemoryStream)
	private
		FExclusive:	Boolean;
		FAllocated:	Boolean;
	protected
    function	GetPrimary:	Boolean;	virtual;	abstract;
		procedure	SetExclusive(Val:Boolean);	virtual;
		procedure	SetAllocated(Val:Boolean);	virtual;
	public
		function	Read(var	Buffer;Count:Longint):	Longint;	override;
		function	Write(const	Buffer;Count:Longint):	Longint;	override;
		procedure	Allocate;	virtual;	abstract;
		procedure	DeAllocate;	virtual;	abstract;
		procedure	Lock(TimeOut:DWord=INFINITE);	virtual;	abstract;
		procedure	Unlock;	virtual;	abstract;
		property	Exclusive:	Boolean	read	FExclusive	write	SetExclusive;
		property	Allocated:	Boolean	read	FAllocated	write	SetAllocated;
		property	Primary:	Boolean	read	GetPrimary;
	end;

type
	TNullStream=class(TShareableMemoryStream)
	protected
    function	GetPrimary:	Boolean;	override;
	public
		procedure	Allocate;	override;
		procedure	DeAllocate;	override;
		procedure	Lock(TimeOut:DWord=INFINITE);	override;
		procedure	Unlock;	override;
  end;

{$IFDEF WINDOWS}
type
	TMappedFileStream=class(TShareableMemoryStream)
	private
		FHandle:	THandle;
		FFileHandle:	THandle;
		FName:	string;
		FSize:	Longint;
		FReadOnly:	Boolean;
		FMutex:	TMutex;
		FPrimary:	Boolean;
	protected
		function	GetPrimary:	Boolean;	override;
		procedure	SetSize(NewSize:Longint);	override;
		procedure	SetName(NewName:string);	virtual;
		procedure	SetFileHandle(Value:THandle);	virtual;
		property	Handle:	THandle	read	FHandle;
	public
		constructor	Create;	overload;
		constructor	Create(Name:string;Size:Longint=0);	overload;
		constructor	Create(hFile:THandle);	overload;
		destructor	Destroy;	override;
		procedure	Allocate;	override;
		procedure	DeAllocate;	override;
		procedure	Lock(TimeOut:DWord=INFINITE);	override;
		procedure	Unlock;	override;
		property	Name:	string	read	FName	write	SetName;
		property	FileHandle:	THandle	read	FFileHandle	write	SetFileHandle;
		property	ReadOnly:	Boolean	read	FReadOnly	write	FReadOnly;
		property	Exclusive;
	end;

type
	TVirtualAllocMemoryStream=class(TShareableMemoryStream)
	private
		FSize:	Integer;
		FProcess:	THandle;
	protected
		procedure	SetSize(NewSize:Longint);	override;
	public
		constructor	Create(Size:Integer=0);
		destructor	Destroy;	override;
		procedure	Allocate;	override;	overload;
    procedure	Allocate(ProcessID:DWord);	virtual;	overload;
		procedure	DeAllocate;	override;
		procedure	Lock(TimeOut:DWord=INFINITE);	override;
		procedure	Unlock;	override;
		function	Read(var	Buffer;Count:Longint):	Longint;	override;
		function	Write(const	Buffer;Count:Longint):	Longint;	override;
		property	Process:	THandle	read	FProcess	write	FProcess;
	end;
{$ENDIF}

type
	TShareableMemoryAdapter=class(TObject)
	private
		FStream:	TShareableMemoryStream;
	protected
		function	GetMemory:	Pointer;
		function	GetSize:	Integer;
		function	GetPosition:	Integer;
		procedure	SetPosition(Val:Integer);
	public
		constructor	Create(Size:Integer=0);
		destructor	Destroy;	override;
		function	Read(var	Buffer;Count:Longint):	Longint;
		function	Write(const	Buffer;Count:Longint):	Longint;
		function	ReadPos(var	Buffer;Count,Pos:Longint):	Longint;
		function	WritePos(const	Buffer;Count,Pos:Longint):	Longint;
		function	Seek(Offset:Longint;Origin:Word):	Longint;
		property	Memory:	Pointer	read	GetMemory;
		property	Size:	Integer	read	GetSize;
		property	Position:	Integer	read	GetPosition	write	SetPosition;
		property	Stream:	TShareableMemoryStream	read	FStream;
	end;

  function	CreateShareableMemoryStream(Size:Integer=0):	TShareableMemoryStream;

type
	EShareableMemoryError=class(EStreamError);

{$IFDEF WINDOWS}
type
	TVirtualAllocExProc=function(hProcess:THandle;lpAddress:Pointer;
		dwSize,flAllocationType:DWORD;flProtect:DWORD):	Pointer;	stdcall;

	TVirtualFreeExProc=function(hProcess:THandle;lpAddress:Pointer;
		dwSize,dwFreeType:DWORD):	Pointer;	stdcall;
{$ENDIF}

implementation

{$IFDEF	WINDOWS}
var
	hLibKernel32:	THandle;
	VirtualAllocExProc:	TVirtualAllocExProc=nil;
	VirtualFreeExProc:	TVirtualFreeExProc=nil;
{$ENDIF}

function	CreateShareableMemoryStream(Size:Integer=0):	TShareableMemoryStream;
begin
{$IFDEF WINDOWS}
	case	Win32Platform	of
		VER_PLATFORM_WIN32_NT:	Result:=TVirtualAllocMemoryStream.Create(Size);
		VER_PLATFORM_WIN32_WINDOWS:	Result:=TMappedFileStream.Create('',Size);
		else
			Result:=TNullStream.Create;
	end;
{$ELSE}
	Result:=TNullStream.Create;
{$ENDIF}
end;

//----------------------------------------------------------------------

{	TCustomLightMemoryStream	}

constructor	TCustomLightMemoryStream.Create(Mem:Pointer;Size:Integer);
begin
	inherited	Create;

	SetPointer(Mem,Size);
end;

constructor	TCustomLightMemoryStream.Create(M:TCustomMemoryStream);
begin
	Create(M.Memory,M.Size);
end;

function	TCustomLightMemoryStream.GetAvailSize:	Integer;
begin
	Result:=Size-Position;
end;

function	TCustomLightMemoryStream.GetAvailable:	Boolean;
begin
	Result:=AvailSize>0;
end;

function	TCustomLightMemoryStream.Getc:	Byte;
begin
	ReadBuffer(Result,SizeOf(Byte));
end;

procedure	TCustomLightMemoryStream.Putc(c:Byte);
begin
	WriteBuffer(c,SizeOf(Byte));
end;

function	TCustomLightMemoryStream.Getw:	Word;
begin
	ReadBuffer(Result,SizeOf(Word));
end;

procedure	TCustomLightMemoryStream.Putw(w:Word);
begin
	WriteBuffer(w,SizeOf(Word));
end;

function	TCustomLightMemoryStream.Getdw:	DWord;
begin
	ReadBuffer(Result,SizeOf(DWord));
end;

procedure	TCustomLightMemoryStream.Putdw(dw:DWord);
begin
	WriteBuffer(dw,SizeOf(DWord));
end;

function	TCustomLightMemoryStream.Write(const	Buffer;Count:Longint):	Longint;
begin
	Assert(Assigned(Memory));

	Result:=Min(Count,Size-Position);
	Move(Buffer,PChar(Memory)[Position],Result);
	Seek(Result,soFromCurrent);
end;

procedure	TCustomLightMemoryStream.Clear(Value:Char);
begin
	Assert(Assigned(Memory));

	FillChar(Memory^,Size,Value);
end;

//----------------------------------------------------------------------

constructor	TLightMemoryStream.Create(Mem:Pointer;Size:Integer);
begin
	inherited;
end;

constructor	TLightMemoryStream.Create(M:TCustomMemoryStream);
begin
	inherited;
end;

//----------------------------------------------------------------------

{$IFDEF WINDOWS}
{	TMemoryHandleStream	}

constructor	TMemoryHandleStream.Create;
begin
	inherited	Create;

	SetPointer(nil,0);
end;

{
constructor	TMemoryHandleStream.Create(Size:Longint);
begin
	inherited	Create;

	SetSize(Size);
end;
}

constructor	TMemoryHandleStream.Create(Handle:HLOCAL);
var
	Size:	Integer;
	Mem:	Pointer;
begin
	inherited	Create;

	FHandle:=Handle;
	Mem:=LocalLock(Handle);
	Size:=LocalSize(Handle);
	SetPointer(Mem,Size);
end;

destructor	TMemoryHandleStream.Destroy;
begin
	if	Handle<>HLOCAL(nil)	then
	begin
		LocalUnlock(Handle);
		LocalFree(Handle);
	end;

	inherited;
end;

function	TMemoryHandleStream.Release:	HLOCAL;
begin
	Result:=FHandle;
	FHandle:=HLOCAL(nil);
	SetPointer(nil,0);
end;

procedure	TMemoryHandleStream.SetSize(NewSize:Longint);
var
	Size:	Integer;
	Mem:	Pointer;
begin
	inherited;

	//ƳƤϤʤ
	if	FHandle<>THandle(nil)	then	Exit;

	FHandle:=LocalAlloc(LMEM_FIXED,NewSize);
	Mem:=LocalLock(FHandle);
	Size:=NewSize;
	SetPointer(Mem,Size);
end;

procedure	TMemoryHandleStream.Clear(Value:Char);
begin
	Assert(Assigned(Memory));

	FillChar(Memory^,Size,Value);
end;
{$ENDIF}

//----------------------------------------------------------------------

{	TShareableMemoryStream	}

function	TShareableMemoryStream.Read(var	Buffer;Count:Longint):	Longint;
begin
	Lock;
	try
		Result:=inherited	Read(Buffer,Count);
	finally
		Unlock;
	end;
end;

function	TShareableMemoryStream.Write(const	Buffer;Count:Longint):	Longint;
begin
	Lock;
	try
		Result:=inherited	Write(Buffer,Count);
	finally
		Unlock;
	end;
end;

procedure	TShareableMemoryStream.SetExclusive(Val:Boolean);
begin
	if	Allocated	then
		raise	EShareableMemoryError.Create('Already allocated!');

	FExclusive:=Val;
end;

procedure	TShareableMemoryStream.SetAllocated(Val:Boolean);
begin
	FAllocated:=Val;
end;

//----------------------------------------------------------------------

{	TNullStream	}
function	TNullStream.GetPrimary:	Boolean;
begin	Result:=False;	end;

procedure	TNullStream.Allocate;
begin	end;

procedure	TNullStream.DeAllocate;
begin	end;

procedure	TNullStream.Lock(TimeOut:DWord=INFINITE);
begin	end;

procedure	TNullStream.Unlock;
begin	end;

//----------------------------------------------------------------------

{$IFDEF WINDOWS}
{	TMappedFileStream	}

constructor	TMappedFileStream.Create;
begin
	inherited	Create;

	FName:='';
	FSize:=0;
	FFileHandle:=THandle(-1);
end;

constructor	TMappedFileStream.Create(Name:string;Size:Longint);
begin
	inherited	Create;

	FName:=Name;
	FSize:=Size;
	FFileHandle:=THandle(-1);
end;

constructor	TMappedFileStream.Create(hFile:THandle);
begin
	inherited	Create;

	FName:='';
	FSize:=0;
	FFileHandle:=hFile;
end;

destructor	TMappedFileStream.Destroy;
begin
	if	Allocated	then	DeAllocate;

	inherited;
end;

procedure	TMappedFileStream.Allocate;
const
	PageProtectMode:	array[Boolean]	of	DWord=(PAGE_READWRITE,PAGE_READONLY);
	FileMapMode:	array[Boolean]	of	DWord=(FILE_MAP_WRITE,FILE_MAP_READ);
var
	Mem:	Pointer;
	ObjName:	string;
	bCreate:	Boolean;
begin
	if	Allocated	then
		raise	EShareableMemoryError.Create('Already allocated!');

	//̾Ĥ
	if	Name=''	then
		Name:=Format('%.8x',[GetTickCount]);

	if	Exclusive	{and	(Name<>'')}	then
	begin
		ObjName:=Name+'::MutexObject';
		FMutex:=TMutex.Create(PChar(ObjName));
	end;

	Lock;
	try
		ObjName:=Name+'::MappedFileObject';

		bCreate:=(FSize>0)	or	(FileHandle<>THandle(-1));

		if	bCreate	then
		begin
			//
			FHandle:=CreateFileMapping(FileHandle,nil,PageProtectMode[ReadOnly],0,FSize,PChar(ObjName));
			if	FHandle=THandle(nil)	then
				raise	EShareableMemoryError.CreateFmt('Can''t create a FMO: %s(%s)',[ObjName,SysErrorMessage(GetLastError)]);
		end	else	begin
			//¸Υޥåץɥե򳫤
			FHandle:=OpenFileMapping(FileMapMode[ReadOnly],False,PChar(ObjName));
			if	FHandle=THandle(nil)	then
				raise	EShareableMemoryError.Create('Can''t open a FMO: '+ObjName);
		end;

		try
			if	bCreate	then
				case	GetLastError	of
					0:
						FPrimary:=True;

					ERROR_ALREADY_EXISTS:
						FPrimary:=False;

					else
						RaiseLastOSError;
				end
			else
				FPrimary:=False;

			Mem:=MapViewOfFile(FHandle,FileMapMode[ReadOnly],0,0,0);
			if	not	Assigned(Mem)	then
				RaiseLastOSError;

			if	(FSize=0)	and	(FileHandle<>0)	then
			begin
				FSize:=GetFileSize(FileHandle,nil);
				if	FSize=Longint(-1)	then
					RaiseLastOSError;
			end;

			SetPointer(Mem,FSize);
			Allocated:=True;
		except
			CloseHandle(FHandle);
			FHandle:=THandle(nil);
			raise;
		end;
	finally
		Unlock;
	end;
end;

procedure	TMappedFileStream.DeAllocate;
begin
	if	not	Allocated	then
		raise	EShareableMemoryError.Create('Already deallocated!');

	if	Assigned(Memory)	then	UnmapViewOfFile(Memory);
	if	FHandle<>0	then	CloseHandle(FHandle);
	if	Exclusive	then	FreeAndNil(FMutex);

	SetPointer(nil,0);
	Allocated:=False;
end;

procedure	TMappedFileStream.Lock(TimeOut:DWord);
begin
	if	Exclusive	and	Allocated	then
	begin
		Assert(Assigned(FMutex));
		FMutex.Lock(TimeOut);
	end;
end;

procedure	TMappedFileStream.Unlock;
begin
	if	Exclusive	and	Allocated	then
	begin
		Assert(Assigned(FMutex));
		FMutex.Unlock;
	end;
end;

function	TMappedFileStream.GetPrimary:	Boolean;	override;
begin
	Result:=FPrimary;
end;

procedure	TMappedFileStream.SetSize(NewSize:Longint);
begin
	inherited;

	if	Allocated	then
		raise	EShareableMemoryError.Create('Can''t modify Size. Already allocated.');

	FSize:=NewSize;
end;

procedure	TMappedFileStream.SetName(NewName:string);
begin
	if	Allocated	then
		raise	EShareableMemoryError.Create('Can''t modify Name. Already allocated.');

	FName:=NewName;
end;

procedure	TMappedFileStream.SetFileHandle(Value:THandle);
begin
	if	Allocated	then
		raise	EShareableMemoryError.Create('Can''t modify FileHandle. Already allocated.');

	FFileHandle:=Value;
end;
{$ENDIF}

//----------------------------------------------------------------------

{$IFDEF WINDOWS}
{	TVirtualAllocMemoryStream	}

constructor	TVirtualAllocMemoryStream.Create(Size:Integer=0);
begin
	if	Assigned(VirtualAllocExProc)	and	Assigned(VirtualFreeExProc)	then
	begin
		inherited;

		FSize:=Size;
	end
	else
		raise	EShareableMemoryError.Create('꤬ͭѤǤޤ(WinNT).');
end;

destructor	TVirtualAllocMemoryStream.Destroy;
begin
	if	Allocated	then	DeAllocate;

	inherited;
end;

procedure	TVirtualAllocMemoryStream.SetSize(NewSize:Longint);
begin
	inherited;

	if	Allocated	then
		raise	EShareableMemoryError.Create('Can''t modify Size. Already allocated.');

	FSize:=NewSize;
end;

procedure	TVirtualAllocMemoryStream.Allocate;
begin
	Allocate(0);
end;

procedure	TVirtualAllocMemoryStream.Allocate(ProcessID:DWord);
var
	Mem:	Pointer;
begin
	if	Allocated	then
		raise	EShareableMemoryError.Create('Already allocated!');

	FProcess := OpenProcess(
		PROCESS_VM_OPERATION or PROCESS_VM_READ or PROCESS_VM_WRITE,
		False,
		ProcessID
	);
	if	FProcess=0	then
		raise	EShareableMemoryError.Create('Can''t open Process.');

	Mem:=VirtualAllocExProc(
		FProcess,
		nil,
		FSize,
		MEM_RESERVE or MEM_COMMIT,
		PAGE_READWRITE
	);

	SetPointer(Mem,FSize);
	Allocated:=True;
end;

procedure	TVirtualAllocMemoryStream.DeAllocate;
begin
	if	not	Allocated	then
		raise	EShareableMemoryError.Create('Already deallocated!');


	if	Assigned(Memory)	then
		VirtualFreeExProc(FProcess,Memory,0,MEM_RELEASE);

	CloseHandle(FProcess);

	SetPointer(nil,0);
	Allocated:=False;
end;

procedure	TVirtualAllocMemoryStream.Lock(TimeOut:DWord=INFINITE);
begin
end;

procedure	TVirtualAllocMemoryStream.Unlock;
begin
end;

function	TVirtualAllocMemoryStream.Read(var	Buffer;Count:Longint):	Longint;
var
	dw:	DWord;
begin
	Count:=Min(Count,Size-Position);
	ReadProcessMemory(FProcess,@PChar(Memory)[Position],@Buffer,Count,dw);
	Result:=dw;
	Seek(Result,soFromCurrent);
end;

function	TVirtualAllocMemoryStream.Write(const	Buffer;Count:Longint):	Longint;
var
	dw:	DWord;
begin
	Count:=Min(Count,Size-Position);
	WriteProcessMemory(FProcess,@PChar(Memory)[Position],@Buffer,Count,dw);
	Result:=dw;
	Seek(Result,soFromCurrent);
end;
{$ENDIF}

//----------------------------------------------------------------------

{	TShareableMemoryAdapter	}

constructor	TShareableMemoryAdapter.Create(Size:Integer=0);
begin
	FStream:=CreateShareableMemoryStream(Size);
	Stream.Allocate;
end;

destructor	TShareableMemoryAdapter.Destroy;
begin
	Stream.Free;
end;

function	TShareableMemoryAdapter.GetMemory:	Pointer;
begin
	Assert(Assigned(FStream));
	Result:=Stream.Memory;
end;

function	TShareableMemoryAdapter.GetSize:	Integer;
begin
	Assert(Assigned(FStream));
	Result:=Stream.Size;
end;

function	TShareableMemoryAdapter.GetPosition:	Integer;
begin
	Assert(Assigned(FStream));
	Result:=Stream.Position;
end;

procedure	TShareableMemoryAdapter.SetPosition(Val:Integer);
begin
	Assert(Assigned(FStream));
	Stream.Position:=Val;
end;

function	TShareableMemoryAdapter.Read(var	Buffer;Count:Longint):	Longint;
begin
	Assert(Assigned(FStream));
	Result:=Stream.Read(Buffer,Count);
end;

function	TShareableMemoryAdapter.Write(const	Buffer;Count:Longint):	Longint;
begin
	Assert(Assigned(FStream));
	Result:=Stream.Write(Buffer,Count);
end;

function	TShareableMemoryAdapter.ReadPos(var	Buffer;Count,Pos:Longint):	Longint;
begin
	Assert(Assigned(FStream));
	Stream.Position:=Pos;
	Result:=Stream.Read(Buffer,Count);
end;

function	TShareableMemoryAdapter.WritePos(const	Buffer;Count,Pos:Longint):	Longint;
begin
	Assert(Assigned(FStream));
	Stream.Position:=Pos;
	Result:=Stream.Write(Buffer,Count);
end;

function	TShareableMemoryAdapter.Seek(Offset:Longint;Origin:Word):	Longint;
begin
	Assert(Assigned(FStream));
	Result:=Stream.Seek(Offset,Origin);
end;

initialization
{$IFDEF WINDOWS}
	hLibKernel32:=LoadLibrary('Kernel32.dll');
	if	hLibKernel32<>0	then
	begin
		VirtualAllocExProc:=TVirtualAllocExProc(GetProcAddress(hLibKernel32,'VirtualAllocEx'));
		VirtualFreeExProc:=TVirtualFreeExProc(GetProcAddress(hLibKernel32,'VirtualFreeEx'));
	end;
{$ENDIF}
finalization
{$IFDEF WINDOWS}
	if	hLibKernel32<>0	then
		FreeLibrary(hLibKernel32);
{$ENDIF}
end.

