unit LinuxExt;

interface

uses
  SysUtils, Types, Classes, Variants, QTypes, QGraphics, QForms,
  Libc, Qt, Xlib, ExtUtils;

{
	Windows, Messages, SysUtils, SysConst, Classes, Graphics, ExtUtils, WinExt;
}

type
	TStopWatch=class(TObject)
	private
		FStartTime:	DWORD;
		FFrequency:	DWORD;
		FEnableFreq:	Boolean;
		FRunning:	Boolean;
	protected
		function	GetTime:	Integer;	virtual;
		property	StartTime:	DWORD	read	FStartTime;
	public
		procedure	Start;
		procedure	Stop;
		procedure	Reset;
		function	IsOver(ATime:Integer):	Boolean;	overload;
		function	IsOver:	Boolean;	overload;
		property	Time:	Integer	read	GetTime;
		property	EnableFreq:	Boolean	read	FEnableFreq	write	FEnableFreq;
		property	Frequency:	DWORD	read	FFrequency	write	FFrequency;
		property	Running:	Boolean	read	FRunning;
	end;

type
{$IFDEF	WINDOWS}
	TWindow=HWND;
{$ELSE}
	TWindow=Xlib.TWindow;
{$ENDIF}

	TViewStyle=(vsNormal,vsHidden,vsMinimized,vsMaximized,vsOther);

	TWindowArray=array[0..0]	of	Window;
	PWindowArray=^TWindowArray;

	TWindowList=class(TList)
	protected
		function Get(Index: Integer): TWindow;
		procedure Put(Index: Integer; Item: TWindow);
	public
		function Add(Item:TWindow):	Integer;
		property Items[Index: Integer]: TWindow read Get write Put; default;
	end;

type
	TWindowParams=class(TObject)
	private
		FHandle:	TWindow;
//		FWindowRect:	TRect;
//		FStyle:	DWORD;
//		FExStyle:	DWORD;
//		FClassStyle:	DWORD;
//		FWindowClass:	string;
//		FModuleName:	string;
	protected
		function	GetAssigned:	Boolean;	virtual;
		procedure	Change;	virtual;
		procedure	SetHandle(Val:TWindow);	virtual;
		function	GetWindowRect:	TRect;	virtual;
		function	GetAsocWindowsRect:	TRect;	virtual;
//		function	GetStyle:	DWORD;	virtual;
//		function	GetExStyle:	DWORD;	virtual;
//		function	GetClassStyle:	DWORD;	virtual;
		function	GetParent:	TWindow;	virtual;
		function	GetRoot:	TWindow;	virtual;
//		function	GetWindowClass:	string;	virtual;
		function	GetWindowText:	string;	virtual;
		procedure	SetWindowText(Val:string);	virtual;
		function	GetModuleName:	string;	virtual;
		function	GetModuleBaseName:	string;	virtual;
		function	GetWindowWidth:	Integer;
		function	GetWindowHeight:	Integer;
		function	GetViewStyle:	TViewStyle;
		procedure	SetViewStyle(Val:TViewStyle);
		function	GetDisplay:	PDisplay;
    function	GetScreen:	PScreen;
	public
		function	IsWindow:	Boolean;
		function	IsVisible:	Boolean;
{
		function	IsMinimized:	Boolean;
		function	IsMaximized:	Boolean;
		function	IsRestored:	Boolean;
}
		function	IsChild(hWindow:TWindow):	Boolean;
		property	Handle:	TWindow	read	FHandle	write	SetHandle;
		property	WindowRect:	TRect	read	GetWindowRect;
		property	Width:	Integer	read	GetWindowWidth;
		property	Height:	Integer	read	GetWindowHeight;
		property	AsocWindowsRect:	TRect	read	GetAsocWindowsRect;
//		property	Style:	DWORD	read	GetStyle;
//		property	ExStyle:	DWORD	read	GetExStyle;
//		property	ClassStyle:	DWORD	read	GetClassStyle;
//		property	WindowClass:	string	read	GetWindowClass;
		property	WindowViewStyle:	TViewStyle	read	GetViewStyle	write	SetViewStyle;
		property	ModuleName:	string	read	GetModuleName;
		property	ModuleBaseName:	string	read	GetModuleBaseName;
		property	WindowText:	string	read	GetWindowText	write	SetWindowText;
		property	Assigned:	Boolean	read	GetAssigned;
		property	Parent:	TWindow	read	GetParent;
		property	Root:	TWindow	read	GetRoot;
    property	Display:	PDisplay	read	GetDisplay;
    property	Screen:	PScreen	read	GetScreen;
	end;

	procedure	ZeroMemory(Mem:Pointer;Count:Integer);
	function	GetTickCount:	Cardinal;
	function	GetWorkAreaRect:	TRect;
	function	IsKeyDown(Key:DWORD):	Boolean;
//	function	SetCursorPos(px,py:Integer):	Boolean;
	function	FlashWindow(Win:TWindow;Enabled:Boolean):	Boolean;

	function	GetParentWindow(Win:TWindow):	TWindow;
	function	GetRootWindow(Display:PDisplay):	TWindow;
  function	GetWindowText(Win:TWindow):	string;
  procedure	SetWindowText(Win:TWindow;Text:string);
  function	IsWindow(Win:TWindow):	Boolean;
  function	GetWindowViewStyle(Win:TWindow):	TViewStyle;
	procedure	SetWindowViewStyle(Win:TWindow;Val:TViewStyle);
	function  ActivateScreenSaver(Active:Boolean):	Boolean;
	function	SetScreenSaver(Enabled:Boolean):	Boolean;
	function  GetNextVisibleWindow(Win:TWindow):	TWindow;
	function	GetWindowRect(Win:TWindow):	TRect;
	function	GetActiveWindow:	TWindow;
	function	SetActiveWindow(Win:TWindow):	Boolean;
	function  GetForegroundWindow:	TWindow;
	function	SetForegroundWindow(Win:TWindow):	Boolean;

implementation

//	function	GetWindowList(Display:PDisplay;Win:TWindow;Atom:TAtom;Windows:TWindowList):	Boolean;
//	function	GetWindowStacking(Display:PDisplay;Win:TWindow;WindowList:TWindowList):	Boolean;

procedure	ZeroMemory(Mem:Pointer;Count:Integer);
begin
	FillChar(Mem^,Count,#0);
end;

function	GetTickCount:	Cardinal;
var
	tv:	TTimeVal;
begin
	gettimeofday(tv,nil);
  Result:=Cardinal(Int64(tv.tv_sec)*1000+tv.tv_usec	div	1000);
end;

function	SetCursorPos(px,py:Integer):	Boolean;
begin
	Mouse.CursorPos:=Point(px,py);
  Result:=True;
end;

function	IsKeyDown(Key:DWORD):	Boolean;
begin
	Result:=False;
end;

function  ActivateScreenSaver(Active:Boolean):	Boolean;
begin
	Result:=False;
//	SendMessage(Handle,WM_SYSCOMMAND,SC_SCREENSAVE,0);
end;

function	SetScreenSaver(Enabled:Boolean):	Boolean;
begin
//		if	SystemParametersInfo(SPI_SETSCREENSAVEACTIVE,Ord(Checked),nil,SPIF_SENDWININICHANGE)=False	then
//			RaiseLastOSError;
	Result:=False;
end;

function	FlashWindow(Win:TWindow;Enabled:Boolean):	Boolean;
begin
	Result:=True;
end;

//------------------------------------------------------------

function	GetAtom(Display:PDisplay;AtomName:string):	TAtom;
begin
	Result:=XInternAtom(Display,PChar(AtomName),Ord(True));
end;

function	GetParentWindow(Win:TWindow):	TWindow;
var
  w:	TWindow;
  pw:	PWindow;
  c:	Cardinal;
begin
	if	XQueryTree(QPaintDevice_x11AppDisplay,Win,@w,@Result,@pw,@c)<>0	then
		XFree(pw);
end;

function	GetRootWindow(Display:PDisplay):	TWindow;
begin
	if	not	Assigned(Display)	then
		Display:=QPaintDevice_x11AppDisplay;
	Result:=XDefaultRootWindow(Display);
end;

{
function	GetRootWindow(Win:TWindow):	TWindow;
var
  w:	TWindow;
  pw:	PWindow;
  c:	Cardinal;
begin
	XQueryTree(QPaintDevice_x11AppDisplay,Win,@Result,@w,@pw,@c);
end;
}

function	GetWindowText(Win:TWindow):	string;

	function	GetUtf8Property(Display:PDisplay;Win:TWIndow;Atom:TAtom):	string;
	var
		AtomUt8String:	TAtom;
		ActualType:	TAtom;
		ActualFormat:	Cardinal;
		NItems:	Cardinal;
		BytesAfter:	Cardinal;
		Prop:	PByteArray;
	begin
		AtomUt8String:=GetAtom(Display,'UTF8_STRING');
		ActualType:=None;
		XGetWindowProperty(
			Display,Win,
			Atom,0,MaxInt,
			Ord(False),AtomUt8String,
			@ActualType,@ActualFormat,@NItems,@BytesAfter,@Prop);
		Result:=PChar(Prop);
	end;

	function	GetTextProperty(Display:PDisplay;Win:TWIndow;Atom:TAtom):	string;
	var
		Prop:	XTextProperty;
		List:	PPCharArray;
		Count:	Cardinal;
	begin
		XGetTextProperty(Display,Win,@Prop,Atom);
		XmbTextPropertyToTextList(Display,@Prop,@List,@Count);
		if	Count>0	then
			Result:=List[0]
		else
			Result:='';
	end;

var
	Display:	PDisplay;
begin
	Display:=QPaintDevice_x11AppDisplay;
	Result:=GetUtf8Property(Display,Win,GetAtom(Display,'_NET_WM_VISIBLE_NAME'));
	if	Result<>''	then	Exit;

	Result:=GetUtf8Property(Display,Win,GetAtom(Display,'_NET_WM_NAME'));
	if	Result<>''	then	Exit;

	Result:=GetTextProperty(Display,Win,XA_WM_NAME);
end;

{
var
	Prop:	XTextProperty;
	P:	PPChar;
	Count:	Integer;
begin
	XGetWMName(QPaintDevice_x11AppDisplay,Win,@Prop);
	XmbTextPropertyToTextList(QPaintDevice_x11AppDisplay,@Prop,@P,@Count);
	if	Assigned(P)	then
	try
		if	Count>0	then
			Result:=P^
		else
			Result:='';
	finally
		XFree(P);
	end;
end;
}

procedure	SetWindowText(Win:TWindow;Text:string);
var
	Prop:	XTextProperty;
  Buffer:	PChar;
begin
	Buffer:=StrAlloc(Length(Text));
  try
		XmbTextListToTextProperty(QPaintDevice_x11AppDisplay,@Buffer,1,XTextStyle,@Prop);
    XSetWMName(QPaintDevice_x11AppDisplay,Win,@Prop);
  finally
  	StrDispose(Buffer);
  end;
end;

function	IsWindow(Win:TWindow):	Boolean;
var
  w:	TWindow;
	pw:	PWindow;
  c:	Cardinal;
begin
	Result:=XQueryTree(QPaintDevice_x11AppDisplay,Win,@w,@w,@pw,@c)<>0;
	if	Result	then
		XFree(pw);
end;

function	IsWindowVisible(Win:TWindow):	Boolean;
var
	Attr:	XWindowAttributes;
begin
	XGetWindowAttributes(QPaintDevice_x11AppDisplay,Win,@Attr);
	Result:=Attr.map_state=IsViewable;
end;

function	GetWindowViewStyle(Win:TWindow):	TViewStyle;
var
	Hints:	PXWMHints;
begin
	Hints:=XGetWMHints(QPaintDevice_x11AppDisplay,Win);
	case	Hints.initial_state	of
  	WithdrawnState:
			Result:=vsHidden;
    NormalState:
			Result:=vsNormal;
    IconicState:
			Result:=vsMinimized;
    ZoomState:
			Result:=vsMaximized;
    else
			Result:=vsOther;
  end;
end;

procedure	SetWindowViewStyle(Win:TWindow;Val:TViewStyle);
begin
	case	Val	of
    vsNormal:;
    vsMinimized:;
    vsMaximized:;
    vsHidden:;
	end;
end;

function	GetWindowPosition(Win:TWindow):	TPoint;
var
	Display:	PDisplay;
	x,y:	Integer;
	Child:	TWindow;
begin
	Display:=QPaintDevice_x11AppDisplay;
	if	XTranslateCoordinates(Display,Win,GetRootWindow(Display),0,0,@x,@y,@Child)=Ord(False)	then
	begin
		x:=0;
		y:=0;
	end;
	Result:=Point(x,y);
end;

function	GetWindowRect(Win:TWindow):	TRect;
var
	Display:	PDisplay;
	Root:	TWindow;
	w,h:	Cardinal;
	BorderWidth, Depth:	Cardinal;
	Pt:	TPoint;
begin
	Display:=QPaintDevice_x11AppDisplay;
	if	XGetGeometry(Display,Win,@Root,@Pt.x,@Pt.y,@w,@h,@BorderWidth,@Depth)=Ord(False)	then
	begin
		w:=0;
		h:=0;
	end;
	Pt:=GetWindowPosition(Win);
	Result:=Bounds(Pt.x,Pt.y,w,h);
end;

{
var
	Attr:	XWindowAttributes;
begin
	XGetWindowAttributes(QPaintDevice_x11AppDisplay,Win,@Attr);
	Result:=Bounds(Attr.x,Attr.y,Attr.width,Attr.height);
end;
}

function	GetWorkAreaRect:	TRect;
begin
	Result:=GetWindowRect(XDefaultRootWindow(QPaintDevice_x11AppDisplay));
end;

function	GetWindowList(Display:PDisplay;Win:TWindow;Atom:TAtom;Windows:TWindowList):	Boolean;
var
	ActualType:	TAtom;
	ActualFormat:	Integer;
	NItems:	Cardinal;
	BytesAfter:	Cardinal;
	data:	PByte;
	i:	Cardinal;
	PWin:	PWindow;
begin
	if	not	Assigned(Display)	then
		Display:=QPaintDevice_x11AppDisplay;

	Result:=False;

	ActualType:=None;
	XGetWindowProperty(
		Display,Win,
		Atom,0,MaxLongint,
		Ord(False),XA_WINDOW,@ActualType,@ActualFormat,
		@NItems,@BytesAfter,@data);
	try
		if	ActualType=XA_WINDOW	then
		begin
			Windows.Clear;
			PWin:=PWindow(data);
			for	i:=0	to	NItems-1	do
			begin
				Windows.Add(PWin^);
				Inc(Pwin);
			end;
			Result:=True;
		end;
	finally
		XFree(data);
	end;
end;

function	GetWindowStacking(Display:PDisplay;Win:TWindow;WindowList:TWindowList):	Boolean;
begin
	if	not	Assigned(Display)	then
		Display:=QPaintDevice_x11AppDisplay;

	Result:=GetWindowList(Display,Win,GetAtom(Display,'_NET_CLIENT_LIST_STACKING'),WindowList);
end;

function	GetActiveWindow:	TWindow;
var
	Display:	PDisplay;
	Atom:	TAtom;
	ActualType:	TAtom;
	ActualFormat:	Integer;
	NItems:	Cardinal;
	BytesAfter:	Cardinal;
	Data:	PByte;
begin
	Display:=QPaintDevice_x11AppDisplay;

	Result:=None;
	Atom:=GetAtom(Display,'_NET_ACTIVE_WINDOW');
	XGetWindowProperty(
		Display,XDefaultRootWindow(Display),
		Atom,0,MaxLongint,
		Ord(False),XA_WINDOW,@ActualType,@ActualFormat,
		@NItems,@BytesAfter,@Data);
	if	Assigned(Data)	then
	try
		Result:=PWindow(Data)^;
	finally
		XFree(Data);
	end;
end;

function  GetForegroundWindow:	TWindow;
var
	Display:	PDisplay;
	WinList:	TWindowList;
begin
	Display:=QPaintDevice_x11AppDisplay;

	WinList:=TWindowList.Create;
	try
		GetWindowStacking(Display,GetRootWindow(Display),WinList);
//		GetWindowList(Display,GetRootWindow(Display),GetAtom(Display,'_NET_CLIENT_LIST_STACKING'),WindowList);
//		Result:=WinList[WinList.Count-1];
		Result:=WinList[0];
	finally
		WinList.Free;
	end;
end;

(*
var
	children:	PWindow;
	count:	Cardinal;
	w:	TWindow;
begin
	Result:=TWindow(nil);

	if	not	Assigned(Display)	then
		Display:=QPaintDevice_x11AppDisplay;

	if	XQueryTree(Display,GetRootWindow(Display),@w,@w,@children,@count)<>0	then
	begin
		if	count>0	then
			Result:=TWindow(PPointerArray(children)[count-1]);
		XFree(children);
	end;
end;
*)

function	SetForegroundWindow(Win:TWindow):	Boolean;
begin
	Result:=XRaiseWindow(QPaintDevice_x11AppDisplay,Win)<>0;
end;

function	SetActiveWindow(Win:TWindow):	Boolean;
var
	E:	XEvent;
	Display:	PDisplay;
begin
	Display:=QPaintDevice_x11AppDisplay;
	with	E	do
	begin
		xclient.xtype:=ClientMessage;
		xclient.serial:=0;
		xclient.send_event:=Ord(True);
		xclient.display:=Display;
		xclient.xwindow:=Win;
		xclient.message_type:=GetAtom(Display,'_NET_ACTIVE_WINDOW');
		xclient.format:=32;
		xclient.data.l[0]:=0;
		xclient.data.l[1]:=0;
		xclient.data.l[2]:=0;
	end;
	Result:=XSendEvent(Display,GetRootWindow(Display),Ord(False),SubstructureRedirectMask	or	SubstructureNotifyMask,@E)<>0;
end;

function  GetNextVisibleWindow(Win:TWindow):	TWindow;
var
	siblings:	PWindow;
  count:	Cardinal;
  i:	Cardinal;
  w:	TWindow;
  Found:	Boolean;
begin
	Result:=TWindow(nil);

	if	XQueryTree(QPaintDevice_x11AppDisplay,GetParentWindow(Win),@w,@w,@siblings,@count)<>0	then
	try
		Found:=False;
		for	i:=0	to	count-1	do
		begin
			w:=siblings^;

			if	Found	then
			begin
				if	IsWindow(w)	then
				begin
					Result:=w;
					Break;
				end
			end
			else	begin
				if	w=Win	then
					Found:=True;
			end;

			Inc(siblings);
		end;
	finally
		XFree(siblings);
	end;
end;

//------------------------------------------------------------
//	TWindowList
//------------------------------------------------------------

function TWindowList.Get(Index: Integer): TWindow;
begin
	Result:=TWindow(inherited	Get(Index));
end;

procedure TWindowList.Put(Index: Integer; Item: TWindow);
begin
	inherited	Put(Index,Pointer(Item));
end;

function TWindowList.Add(Item:TWindow):	Integer;
begin
	Result:=inherited	Add(Pointer(Item));
end;

//------------------------------------------------------------
//  TStopWatch
//------------------------------------------------------------

procedure	TStopWatch.Start;
begin
	FStartTime:=GetTickCount;
	FRunning:=True;
end;

procedure	TStopWatch.Stop;
begin
	FRunning:=False;
end;

procedure	TStopWatch.Reset;
begin
	FStartTime:=GetTickCount;
end;

function	TStopWatch.GetTime:	Integer;
begin
	if	Running	then
		Result:=GetTickCount-StartTime
	else
		Result:=0;
end;

function	TStopWatch.IsOver(ATime:Integer):	Boolean;
begin
	Result:=Running	and	(GetTime>ATime);

	if	Result	and	EnableFreq	then
		Inc(FStartTime,ATime);
end;

function	TStopWatch.IsOver:	Boolean;
begin
	if	EnableFreq	then
		Result:=IsOver(Frequency)
	else
		Result:=False;
end;

//------------------------------------------------------------
{	TWindowParams	}
//------------------------------------------------------------

function	TWindowParams.GetAssigned:	Boolean;
begin
	Result:=(Handle<>TWindow(nil));
end;

procedure	TWindowParams.Change;
begin
end;

procedure	TWindowParams.SetHandle(Val:TWindow);
begin
	FHandle:=Val;
	Change;
end;

function	TWindowParams.GetWindowRect:	TRect;
begin
	Result:=LinuxExt.GetWindowRect(Handle)
end;

function	TWindowParams.GetAsocWindowsRect:	TRect;
begin
	Result:=GetWindowRect;
end;

function	TWindowParams.GetParent:	TWindow;
begin
	Result:=GetParentWindow(Handle);
end;

function	TWindowParams.GetRoot:	TWindow;
begin
	Result:=GetRootWindow(Display);
end;

function	TWindowParams.GetWindowText:	string;
begin
	if	IsWindow	then
		Result:=LinuxExt.GetWindowText(Handle)
	else
		Result:='';
end;

procedure	TWindowParams.SetWindowText(Val:string);
begin
	if	IsWindow	then
		LinuxExt.SetWindowText(Handle,Val);
end;

function	TWindowParams.GetModuleName:	string;
begin
	Result:='';
end;

function	TWindowParams.GetModuleBaseName:	string;
begin
	Result:='';
end;

function	TWindowParams.GetWindowWidth:	Integer;
begin
	Result:=0;
end;

function	TWindowParams.GetWindowHeight:	Integer;
begin
	Result:=0;
end;

function	TWindowParams.GetViewStyle:	TViewStyle;
begin
	Result:=GetWindowViewStyle(Handle);
end;

procedure	TWindowParams.SetViewStyle(Val:TViewStyle);
begin
	SetWindowViewStyle(Handle,Val);
end;

{
function	TWindowParams.IsMinimized:	Boolean;
begin
	Result:=GetWindowViewStyle(Handle)=vsMinimized;
end;

function	TWindowParams.IsMaximized:	Boolean;
begin
	Result:=GetWindowViewStyle(Handle)=vsMaximized;
end;

function	TWindowParams.IsRestored:	Boolean;
begin
	Result:=False;
end;
}

function	TWindowParams.IsWindow:	Boolean;
begin
	Result:=LinuxExt.IsWindow(Handle);
end;

function	TWindowParams.IsVisible:	Boolean;
begin
	Result:=IsWindowVisible(Handle);
end;

function	TWindowParams.IsChild(hWindow:TWindow):	Boolean;
begin
	Result:=False;
end;

function	TWindowParams.GetDisplay:	PDisplay;
begin
	Result:=QPaintDevice_x11AppDisplay;
end;

function	TWindowParams.GetScreen:	PScreen;
begin
	Result:=XDefaultScreenOfDisplay(Display);
end;

end.

