unit WidgetShape;

interface

uses
	SysUtils, Types, Classes, Variants, QTypes, QGraphics, QControls, QForms,
	QDialogs, QStdCtrls, QImgList, QExtCtrls, QMenus, Math, Qt, LinuxExt;

type
	TWidgetShape=class(TObject)
	private
		FBitmap:	TBitmap;
		FWidget:	TWidgetControl;
		FRect:	TRect;
	protected
		procedure	UpdateBounds;
		function	GetBitmap:	TBitmap;
		function	GetRect:	TRect;
	public
		constructor	Create(AOwner:TWidgetControl);
		destructor	Destroy;	override;
		procedure	Clear;
		procedure	Shape;
		procedure	Unshape;
		property Rect:	TRect	read	GetRect;
		property	Bitmap:	TBitmap	read	GetBitmap;
  end;

implementation

constructor	TWidgetShape.Create(AOwner:TWidgetControl);
begin
	FBitmap:=TBitmap.Create;
  FWidget:=AOwner;
  UpdateBounds;
end;

destructor	TWidgetShape.Destroy;
begin
	FBitmap.Free;
end;

procedure	TWidgetShape.UpdateBounds;
begin
	if	(FBitmap.Width<>FWidget.ClientWidth)	or
			(FBitmap.Height<>FWidget.ClientHeight)	then
	begin
		FBitmap.Width:=FWidget.ClientWidth;
		FBitmap.Height:=FWidget.ClientHeight;
		Clear;
	end;
end;

function	TWidgetShape.GetRect:	TRect;
begin
//	FRect:=GetWindowRect(QWidget_winId(FWidget.Handle));

//	OffsetRect(FRect,-FRect.Left,-FRect.Right);
	Result:=FRect;

//	Result:=Bounds(0,0,100,100);
end;

function	TWidgetShape.GetBitmap:	TBitmap;
begin
	UpdateBounds;
  Result:=FBitmap;
end;

procedure	TWidgetShape.Shape;
begin
	FWidget.ClientWidth:=FBitmap.Width;
	FWidget.ClientHeight:=FBitmap.Height;
	FBitmap.Mask(clMask);
	QWidget_setMask(FWidget.Handle, QPixmap_Mask(FBitmap.Handle));
	FRect:=Bounds(0,0,FBitmap.Width,FBitmap.Height);
end;

procedure	TWidgetShape.Unshape;
begin
	Clear;
  Shape;
end;

procedure	TWidgetShape.Clear;
begin
	FBitmap.Canvas.Brush.Color:=clDontMask;
	FBitmap.Canvas.FillRect(Classes.Rect(0,0,FBitmap.Width,FBitmap.Height));
end;

end.

