unit Alarm;

interface

uses
	SysUtils, Types, Classes, Variants, QTypes, ExtUtils, Xlib;

{
	Windows, SysUtils, Classes, MMSystem, ExtUtils;
}

type
	EThreadTerminated=class(Exception);

	TAlarmKind=(akNone,akCount,akTimer,akPegion);
	TFreqLevel=(flCustom,flSlow,flNormal,flFast);

type
	TAlarmParams=record
		Kind:	TAlarmKind;
		Count:	Integer;
		Freq:	Integer;
		FreqLevel:	TFreqLevel;
		HalfHourAlarm:	Boolean;
		FileName:	string;
	end;
	PAlarmParams=^TAlarmParams;

type
	TAlarmThread = class(TThread)
	private
		FField:	TAlarmParams;
	protected
		procedure	CheckTerminate;	virtual;
		procedure	DoAlarm;	virtual;	abstract;
		procedure Execute; override;
		function	GetKind:	TAlarmKind;
		function	GetCount:	Integer;
		function	GetFreq:	Integer;
		function	GetFreqLevel:	TFreqLevel;
		function	GetFileName:	string;
		procedure	SetKind(Val:TAlarmKind);
		procedure	SetCount(Val:Integer);
		procedure	SetFreq(Val:Integer);
		procedure	SetFreqLevel(Val:TFreqLevel);
		procedure	SetFileName(Val:string);
	public
		constructor	Create;	overload;
		constructor	Create(Params:TAlarmParams;TerminateProc:TNotifyEvent=nil);	overload;
		property	Kind:	TAlarmKind	read	GetKind	write	SetKind;
		property	Count:	Integer	read	GetCount	write	SetCount;
		property	Freq:	Integer	read	GetFreq	write	SetFreq;
		property	FreqLevel:	TFreqLevel	read	GetFreqLevel	write	SetFreqLevel;
		property	FileName:	string	read	GetFileName	write	SetFileName;
	end;

	TAlarmThreadClass=class	of	TAlarmThread;

type
	TBlinkerThread=class(TAlarmThread)
	protected
		procedure	DoAlarm;	override;
	end;

	TShakerThread=class(TAlarmThread)
	protected
		procedure	DoAlarm;	override;
	end;

	TPlayerThread=class(TAlarmThread)
	protected
		procedure	DoAlarm;	override;
		procedure Execute;	override;
	end;

const
	DefAlarmCount:	array[TAlarmKind]	of	Integer=(0,10,3000,0);
	BlinkerFreqs:	array[TFreqLevel]	of	Integer=(0,1000,100,10);
	ShakerFreqs:	array[TFreqLevel]	of	Integer=(0,250,50,10);
	PlayerFreqs:	array[TFreqLevel]	of	Integer=(0,1000,600,200);

implementation

//------------------------------------------------------------

uses
	LinuxExt;
{	WinExt;	}

{	TAlarmThread	}

type
	HWND=TWindow;

constructor	TAlarmThread.Create;
begin
	FreeOnTerminate:=True;
	Freq:=100;

	inherited	Create(True);
end;

constructor	TAlarmThread.Create(Params:TAlarmParams;TerminateProc:TNotifyEvent=nil);
begin
	FreeOnTerminate:=True;
	FField:=Params;
	OnTerminate:=TerminateProc;

	inherited	Create(False);
end;

procedure	TAlarmThread.CheckTerminate;
begin
	if	Terminated	then
		raise	EThreadTerminated.Create('');
end;

procedure TAlarmThread.Execute;
var
	i:	Integer;
	e:	DWord;
begin
	if	IsCpuBusy	then	Exit;

	try
		case	Kind	of
		akCount,akPegion:
			for	i:=1	to	Count	do
				DoAlarm;

		akTimer:
			begin
				e:=GetTickCount+DWord(Count);
				while	GetTickCount<e	do
					DoAlarm;
			end;
		end;
	except
		on	EThreadTerminated	do;
	end;
end;

function	TAlarmThread.GetKind:	TAlarmKind;
begin
	Result:=FField.Kind;
end;

function	TAlarmThread.GetCount:	Integer;
begin
	Result:=FField.Count;
end;

function	TAlarmThread.GetFreq:	Integer;
begin
	Result:=FField.Freq;
end;

function	TAlarmThread.GetFreqLevel:	TFreqLevel;
begin
	Result:=FField.FreqLevel;
end;

function	TAlarmThread.GetFileName:	string;
begin
	Result:=FField.FileName;
end;

procedure	TAlarmThread.SetKind(Val:TAlarmKind);
begin
	FField.Kind:=Val;
end;

procedure	TAlarmThread.SetCount(Val:Integer);
begin
	FField.Count:=Val;
end;

procedure	TAlarmThread.SetFreq(Val:Integer);
begin
	FField.Freq:=Val;
end;

procedure	TAlarmThread.SetFreqLevel(Val:TFreqLevel);
begin
	FField.FreqLevel:=Val;
end;

procedure	TAlarmThread.SetFileName(Val:string);
begin
	FField.FileName:=Val;
end;

//------------------------------------------------------------

{	TBlinkerThread	}

procedure	TBlinkerThread.DoAlarm;
var
	hWindow:	HWND;
begin
	hWindow:=GetForegroundWindow;
	if	hWindow=HWND(nil)	then	Exit;

	if	FreqLevel<>flCustom	then
		Freq:=BlinkerFreqs[FreqLevel];

	if	Freq<=0	then	Freq:=BlinkerFreqs[flNormal];

	CheckTerminate;
	FlashWindow(hWindow,True);
	Sleep(Freq);
	FlashWindow(hWindow,False);
	CheckTerminate;

	Sleep(Freq);
end;

//------------------------------------------------------------

{	TShakerThread	}

procedure	TShakerThread.DoAlarm;

	procedure	MoveWindow(hWindow:HWND;x,y:Integer);
	begin
//		SetWindowPos(hWindow,0,x,y,0,0,SWP_NOACTIVATE	or	SWP_NOSIZE	or	SWP_NOZORDER);
	end;

const
	cx=1;
	cy=1;
var
	R:	TRect;
	i:	Integer;
	hWindow:	HWND;
begin
	hWindow:=GetForegroundWindow;
	if	hWindow=HWND(nil)	then	Exit;

	CheckTerminate;

	if	FreqLevel<>flCustom	then
		Freq:=ShakerFreqs[FreqLevel];

	if	Freq<=0	then	Freq:=ShakerFreqs[flNormal];

	R:=GetWindowRect(hWindow);
//	GetWindowRect(hWindow,R);
	with	R	do
	begin
		try
			for	i:=0	to	3	do
			begin
				CheckTerminate;
//				case	Random(4)	of
				case	i	of
					0:	MoveWindow(hWindow,Left-cx,Top-cy);
					1:	MoveWindow(hWindow,Left+cx,Top-cy);
					2:	MoveWindow(hWindow,Left+cx,Top+cy);
					3:	MoveWindow(hWindow,Left-cx,Top+cy);
				end;

				Sleep(Freq);
			end;
		finally
			MoveWindow(hWindow,Left,Top);
		end;
	end;
end;

//------------------------------------------------------------

{	TPlayerThread	}

procedure	TPlayerThread.DoAlarm;
//var
//	Flag:	DWord;
begin
{
	if	FreqLevel<>flCustom	then
		Freq:=PlayerFreqs[FreqLevel];

	if	Freq<=0	then	Freq:=PlayerFreqs[flNormal];

	Flag:=SND_FILENAME	or	SND_ASYNC	or	SND_NODEFAULT;
	if	Kind=akTimer	then	Flag:=Flag	or	SND_LOOP;
}

	CheckTerminate;
//	if	not	PlaySound(PChar(FileName),0,Flag)	then
//		Terminate;
end;

procedure TPlayerThread.Execute;
var
	i:	Integer;
	e:	DWord;
begin
	if	IsCpuBusy	then	Exit;

	try
		case	Kind	of
		akCount,akPegion:
			for	i:=1	to	Count	do
			begin
				DoAlarm;
				Sleep(Freq);
			end;

		akTimer:
			begin
				e:=GetTickCount+DWord(Count);
				DoAlarm;
				try
					while	GetTickCount<e	do
					begin
						CheckTerminate;
						Sleep(1000);
					end;
				finally
//					PlaySound(nil,0,0);
				end;
			end;
		end;
	except
		on	EThreadTerminated	do;
	end;
end;

end.

