unit MainDoc;

interface

uses
  SysUtils, Types, Classes, Variants, QTypes, QGraphics, QControls, QForms,
  QDialogs, QStdCtrls, QImgList, QExtCtrls, QMenus, contnrs,
	Document, ExtUtils, LinuxExt, Mascot, Alarm, {SNTPThrd, QRek, }Qt;

(*
	Windows, Messages, SysUtils, Classes, Graphics, Forms, 	Dialogs, Controls,
	Menus, contnrs, ShlObj,
	Document, WinExt, ExtUtils, WinUtils, ShlUtils, SpiMngr, FreeRes,
	Mascot, Alarm, SNTPThrd, QRek, Lexer, Parser;
*)

type
	TIconState=(isNone,isLarge,isSmall);

	TDisableFollowing=(dfDesktop,dfTopMost,dfMaximum,dfFarAway,dfRegion,dfTaskBar);
	TDisableFollowings=set	of	TDisableFollowing;

	TNTPInterval=(ntpiPerHour,ntpiPerDay,ntpiPerWeek,ntpiPerMonth);

	TPositionChangedEvent=procedure(Sender:TObject;x,y:Integer)	of	object;
	TViewStyleChangeEvent=procedure(Sender:TObject;ViewStyle:TViewStyle)	of	object;

type
	TFolderMenuOpt=(fmoEnabled,fmoSubMenu,fmoRecursive);
	TFolderMenuOpts=set	of	TFolderMenuOpt;

{
	TFolderMenuItem=class(TObject)
	private
		FNumber:	Integer;
		FOpts:	TFolderMenuOpts;
		FImageIndex:	Integer;
		FDisplayName:	string;
		FIdent:	string;
//		FItemIDList:	PItemIDList;
	protected
		constructor	Create;	overload;
		function	GetEnabled:	Boolean;
		procedure	SetEnabled(Val:Boolean);
		function	GetSubMenu:	Boolean;
		procedure	SetSubMenu(Val:Boolean);
		function	GetSubMenuRecursive:	Boolean;
		procedure	SetSubMenuRecursive(Val:Boolean);
	public
		constructor	Create(Id:string;nFolder:Integer;Enabled:Boolean=False);	overload;
		destructor	Destroy;	override;
		procedure	Assign(Item:TFolderMenuItem);
		function	Clone:	TFolderMenuItem;
		property	Ident:	string	read	FIdent;
		property	Number:	Integer	read	FNumber;
		property	Opts:	TFolderMenuOpts	read	FOpts	write	FOpts;
		property	ImageIndex:	Integer	read	FImageIndex;
		property	DisplayName:	string	read	FDisplayName;
//		property	ItemIDList:	PItemIDList	read	FItemIDList;
		property	Enabled:	Boolean	read	GetEnabled	write	SetEnabled;
		property	SubMenu:	Boolean	read	GetSubMenu	write	SetSubMenu;
		property	SubMenuRecursive:	Boolean	read	GetSubMenuRecursive	write	SetSubMenuRecursive;
	end;

	TFolderMenuList=class(TObject)
	private
		FList:	TObjectList;
	protected
		function	GetItems(Idx:Integer):	TFolderMenuItem;
		function	GetCount:	Integer;
	public
		constructor	Create;
		destructor	Destroy;	override;
		procedure	Add(Item:TFolderMenuItem);	overload;
		procedure	Add(S:string;nFolder:Integer;Enabled:Boolean);	overload;
		procedure	Clear;
		procedure	Assign(AList:TFolderMenuList);
		procedure	Exchange(Index1,Index2:Integer);
		property	Items[Idx:Integer]:	TFolderMenuItem	read	GetItems;	default;
		property	Count:	Integer	read	GetCount;
	end;
}

	TCondFlag=(cfFollowing,cfDefButton,cfCaption,cfAsocWindows,cfDialog);
	TCondFlags=set	of	TCondFlag;

	TCondListItem=record
		Name:	string;
		WindowClass:	string;
		WindowText:	string;
		ExeName:	string;
		Width:	Integer;
		Height:	Integer;
		Flags:	TCondFlags;
		Following:	Boolean;
		DefButton:	Boolean;
		CaptionFmt:	Boolean;
		AsocWindows:	Boolean;
		Dialog:	Boolean;
	end;
	PCondListItem=^TCondListItem;

	TCondList=class(TList)
	protected
		function Get(Index:Integer):	PCondListItem;
		procedure Put(Index:Integer;Item:PCondListItem);
	public
		procedure	Assign(Source:TCondList);
		procedure Clear; override;
		function Add(const	Item:TCondListItem): Integer;
		procedure Delete(Index: Integer);
		function IndexOf(Name:string): Integer;
		property Items[Index:Integer]: PCondListItem read Get write Put; default;
	end;

type
	TDocField1=record
		TopMost:	Boolean;
		AlwaysInScreen:	Boolean;
		ZOrderControl:	Boolean;
		TimerInterval:	Integer;
		ChangeInterval:	Integer;
		InitialChange:	Boolean;

		DefButtonEnable:	Boolean;	//*
		UseCaptionFmt:	Boolean;	//*
		UnionAsocWindows:	Boolean;	//*
		EnableActivateModalDialog:	Boolean;	//*
		SensorIDE:	Boolean;

		CaptionFmt:	string;
		SmartMove:	Boolean;
		UsePlugin:	Boolean;
		DisableFollowings:	TDisableFollowings;
		AllowDuplication:	Boolean;
		ShowHint:	Boolean;
		HintColor:	TColor;
		HideByFullScreenWindow:	Boolean;
		PluginDirs:	string;
//		BlinkerParams:	TAlarmParams;
//		ShakerParams:	TAlarmParams;
//		PlayerParams:	TAlarmParams;
		TimeOffset:	Integer;
		UseDesktop:	Boolean;
		IconState:	TIconState;
		NTPServer:	string;
		NTPTimeout:	Integer;
		NTPInterval:	TNTPInterval;
		NTPLastUpdate:	TDateTime;
		NTPSyncPeriodic:	Boolean;
		NTPSyncSystem:	Boolean;
		NTPSyncGradually:	Boolean;
		NTPOnlyIfConnected:	Boolean;
		NTPServerValid:	Boolean;
	end;

type
	TWindowParamsEx=class(TWindowParams)
	private
		FOriginalWindowText:	string;
		FModifiedWindowText:	string;
//		FPreview:	Boolean;
	protected
		procedure	Change;	override;
		procedure	SetHandle(Val:TWindow);	override;
//		function	GetWindowClass:	string;	override;
		function	GetWindowText:	string;	override;
//		procedure	SetWindowText(Val:string);	override;
		function	GetModuleName:	string;	override;
		function	GetModuleBaseName:	string;	override;
		function	GetModifiedWindowText:	string;
		procedure	SetModifiedWindowText(Val:string);
		procedure	UpdateTitle;
	public
		destructor	Destroy;	override;
		procedure	RecoverCaption;
//		function	GetWindowTextOrg:	string;
//		property	WindowTextOrg:	string	read	GetWindowTextOrg;
//		property	Preview:	Boolean	read	FPreview	write	FPreview;
		property	ModifiedWindowText:	string	read  GetModifiedWindowText	write	SetModifiedWindowText;
	end;

type
	TDocForm1=class(TDocument)
	private
		FField:	TDocField1;
		FMascotList:	TMascotList;
		FCondList:	TCondList;
		FWindowParams:	TWindowParamsEx;
//		FLex: TLex;
//		FYacc: TYacc;
		FWindowFollowing:	Boolean;
		FDisableCaption:	Boolean;
		FCursorPos:	TPoint;
		FIsCursorPosStored:	Boolean;
		FDefButtonDone:	Boolean;
//		FhwndProgman:	HWND;
		FLastTime:	TDateTime;
//		FBlinkerThread:	TAlarmThread;
//		FShakerThread:	TAlarmThread;
//		FPlayerThread:	TAlarmThread;
//		FNTPThread:	TSNTPThread;
		FForm2Rect:	TRect;
		FTimerState:	Boolean;
//		FFolderMenuList:	TFolderMenuList;
		FTimeDifference:	Real;
{
		FACColor:	TColorRef;
		FACFont:	TLogFont;
		FACFontEnable:	Boolean;
		FOrgACColor:	TColorRef;
		FOrgACFont:	TLogFont;
		FACFontModified:	Boolean;
}
		FOnPositionChanged:	TPositionChangedEvent;
		FOnWindowChanged:	TNotifyEvent;
		FOnViewStyleChange:	TViewStyleChangeEvent;
	protected
		function	GetTopMost:	Boolean;
		function	GetAlwaysInScreen:	Boolean;
		function	GetZOrderControl:	Boolean;
		function	GetTimerInterval:	Integer;
		function	GetChangeInterval:	Integer;
		function	GetInitialChange:	Boolean;
		function	GetUseCaptionFmt:	Boolean;
		function	GetCaptionFmt:	string;
		function	GetDefButtonEnable:	Boolean;
		function	GetSmartMove:	Boolean;
		function	GetUsePlugin:	Boolean;
		function	GetDisableFollowings:	TDisableFollowings;
		function	GetAllowDuplication:	Boolean;
		function	GetShowHint:	Boolean;
		function	GetHintColor:	TColor;
		function	GetUnionAsocWindows:	Boolean;
		function	GetHideByFullScreenWindow:	Boolean;
		function	GetEnableActivateModalDialog:	Boolean;
		function	GetSensorIDE:	Boolean;
		function	GetPluginDirs:	string;
//		function	GetBlinkerParams:	TAlarmParams;
//		function	GetShakerParams:	TAlarmParams;
//		function	GetPlayerParams:	TAlarmParams;
		function	GetTimeOffset:	Integer;
		function	GetUseDesktop:	Boolean;
		function	GetIconState:	TIconState;
		function	GetNTPServer:	string;
		function	GetNTPTimeout:	Integer;
		function	GetNTPInterval:	TNTPInterval;
		function	GetNTPLastUpdate:	TDateTime;
		function	GetNTPSyncPeriodic:	Boolean;
		function	GetNTPSyncSystem:	Boolean;
		function	GetNTPSyncGradually:	Boolean;
		function	GetNTPOnlyIfConnected:	Boolean;
		function	GetNTPServerValid:	Boolean;

		procedure	SetTopMost(Val:Boolean);
		procedure	SetAlwaysInScreen(Val:Boolean);
		procedure	SetZOrderControl(Val:Boolean);
		procedure	SetTimerInterval(Val:Integer);
		procedure	SetChangeInterval(Val:Integer);
		procedure	SetInitialChange(Val:Boolean);
		procedure	SetUseCaptionFmt(Val:Boolean);
		procedure	SetCaptionFmt(Val:string);
		procedure	SetDefButtonEnable(Val:Boolean);
		procedure	SetSmartMove(Val:Boolean);
		procedure	SetUsePlugin(Val:Boolean);
		procedure	SetDisableFollowings(Val:TDisableFollowings);
		procedure	SetAllowDuplication(Val:Boolean);
		procedure	SetShowHint(Val:Boolean);
		procedure	SetHintColor(Val:TColor);
		procedure	SetUnionAsocWindows(Val:Boolean);
		procedure	SetHideByFullScreenWindow(Val:Boolean);
		procedure	SetEnableActivateModalDialog(Val:Boolean);
		procedure	SetSensorIDE(Val:Boolean);
		procedure	SetPluginDirs(Val:string);
//		procedure	SetBlinkerParams(Val:TAlarmParams);
//		procedure	SetShakerParams(Val:TAlarmParams);
//		procedure	SetPlayerParams(Val:TAlarmParams);
		procedure	SetTimeOffset(Val:Integer);
		procedure	SetUseDesktop(Val:Boolean);
		procedure	SetIconState(Val:TIconState);
		procedure	SetNTPServer(Val:string);
		procedure	SetNTPTimeout(Val:Integer);
		procedure	SetNTPInterval(Val:TNTPInterval);
		procedure	SetNTPLastUpdate(Val:TDateTime);
		procedure	SetNTPSyncPeriodic(Val:Boolean);
		procedure	SetNTPSyncSystem(Val:Boolean);
		procedure	SetNTPSyncGradually(Val:Boolean);
		procedure	SetNTPOnlyIfConnected(Val:Boolean);
		procedure	SetNTPServerValid(Val:Boolean);

		function	GetCurrentMascot:	Integer;
		procedure	SetCurrentMascot(Index:Integer);
		function	GetMascotAnchor:	TAnchor;
		function	GetWindowAnchor:	TAnchor;
		function	GetXPos:	Integer;
		function	GetYPos:	Integer;
		procedure	SetWindowFollowing(Val:Boolean);
		function	GetModifiedNow:	TDateTime;
		function	GetModifiedDate:	TDateTime;
		function	GetModifiedTime:	TDateTime;

		procedure	SetProperty(var	Prop:Boolean;NewVal:Boolean);	overload;
		procedure	SetProperty(var	Prop:Integer;NewVal:Integer);	overload;
		procedure	SetProperty(var	Prop:string;NewVal:string);	overload;

		procedure	SnapToDefButton;
		procedure	RecoverCursorPos;
		procedure	ValidateCursorPos;
		procedure	SaveCursorPos(pt:TPoint);
		procedure	UpdateCaption;
		procedure	UpdateDisableCaption;
		procedure	PositionChanged(x,y:Integer);
		procedure	ViewStyleChange(ViewStyle:TViewStyle);
{
		procedure	ThreadDone(Sender:TObject);
		procedure	UpdateACFont;
		procedure	StoreACFont;
		procedure	RestoreACFont;
}
	public
		constructor	Create(View:TCustomForm);
		destructor	Destroy;	override;
		procedure	Assign(Source:TPersistent);	override;
		procedure	BeginUpdate;	override;
		procedure	EndUpdate;	override;
		procedure	LoadIniFile;	override;
		procedure	SaveIniFile;	override;
		property	IniFile;

		procedure	Timer1Timer;
		procedure	Timer2Timer;
		procedure	TimeChanged;
		procedure	InitFolderMenuList;
		procedure	RandomChange(IgnoreCurrent:Boolean);
//		function	IsCondOk(WindowParams:TWindowParams;Query:TCondFlag):	Boolean;
		function	AllowFollowing:	Boolean;
//		function	GetAsocWindowsRect(hWindow:HWND):	TRect;
		procedure	UpdatePosition(var	pt:TPoint;const	MascotRect:TRect);
		procedure	MascotListChange(Sender:TObject);
//		function	SetTimeDifference(Val:Real):	Boolean;

		property	Field:	TDocField1	read	FField	write	FField;
		property	MascotList:	TMascotList	read	FMascotList;
		property	CondList:	TCondList	read	FCondList;
//		property	Lex:	TLex	read	FLex;
//		property	Yacc:	TYacc	read	FYacc;

		property	TopMost:	Boolean	read	GetTopMost	write	SetTopMost;
		property	AlwaysInScreen:	Boolean	read	GetAlwaysInScreen	write	SetAlwaysInScreen;
		property	ZOrderControl:	Boolean	read	GetZOrderControl	write	SetZOrderControl;
		property	TimerInterval:	Integer	read	GetTimerInterval	write	SetTimerInterval;
		property	ChangeInterval:	Integer	read	GetChangeInterval	write	SetChangeInterval;
		property	InitialChange:	Boolean	read	GetInitialChange	write	SetInitialChange;
		property	UseCaptionFmt:	Boolean	read	GetUseCaptionFmt	write	SetUseCaptionFmt;
		property	CaptionFmt:	string	read	GetCaptionFmt	write	SetCaptionFmt;
		property	DefButtonEnable:	Boolean	read	GetDefButtonEnable	write	SetDefButtonEnable;
		property	SmartMove:	Boolean	read	GetSmartMove	write	SetSmartMove;
		property	UsePlugin:	Boolean	read	GetUsePlugin	write	SetUsePlugin;
		property	DisableFollowings:	TDisableFollowings	read	GetDisableFollowings	write	SetDisableFollowings;
		property	AllowDuplication:	Boolean	read	GetAllowDuplication	write	SetAllowDuplication;
		property	ShowHint:	Boolean	read	GetShowHint	write	SetShowHint;
		property	HintColor:	TColor	read	GetHintColor	write	SetHintColor;
		property	UnionAsocWindows:	Boolean	read	GetUnionAsocWindows	write	SetUnionAsocWindows;
		property	HideByFullScreenWindow:	Boolean	read	GetHideByFullScreenWindow	write	SetHideByFullScreenWindow;
		property	EnableActivateModalDialog:	Boolean	read	GetEnableActivateModalDialog	write	SetEnableActivateModalDialog;
		property	SensorIDE:	Boolean	read	GetSensorIDE	write	SetSensorIDE;
		property	PluginDirs:	string	read	GetPluginDirs	write	SetPluginDirs;
//		property	BlinkerParams:	TAlarmParams	read	GetBlinkerParams	write	SetBlinkerParams;
//		property	ShakerParams:	TAlarmParams	read	GetShakerParams	write	SetShakerParams;
//		property	PlayerParams:	TAlarmParams	read	GetPlayerParams	write	SetPlayerParams;
		property	TimeOffset:	Integer	read	GetTimeOffset	write	SetTimeOffset;
		property	UseDesktop:	Boolean	read	GetUseDesktop	write	SetUseDesktop;
		property	IconState:	TIconState	read	GetIconState	write	SetIconState;
		property	NTPServer:	string	read	GetNTPServer	write	SetNTPServer;
		property	NTPTimeout:	Integer	read	GetNTPTimeout	write	SetNTPTimeout;
		property	NTPInterval:	TNTPInterval	read	GetNTPInterval	write	SetNTPInterval;
		property	NTPLastUpdate:	TDateTime	read	GetNTPLastUpdate	write	SetNTPLastUpdate;
		property	NTPSyncPeriodic:	Boolean	read	GetNTPSyncPeriodic	write	SetNTPSyncPeriodic;
		property	NTPSyncSystem:	Boolean	read	GetNTPSyncSystem	write	SetNTPSyncSystem;
		property	NTPSyncGradually:	Boolean	read	GetNTPSyncGradually	write	SetNTPSyncGradually;
		property	NTPOnlyIfConnected:	Boolean	read	GetNTPOnlyIfConnected	write	SetNTPOnlyIfConnected;
		property	NTPServerValid:	Boolean	read	GetNTPServerValid	write	SetNTPServerValid;

		property	CurrentMascot:	Integer	read	GetCurrentMascot	write	SetCurrentMascot;
		property	CursorPos:	TPoint	read	FCursorPos;
		property	IsCursorPosStored:	Boolean	read	FIsCursorPosStored	write	FIsCursorPosStored;
		property	DefButtonDone:	Boolean	read	FDefButtonDone	write	FDefButtonDone;
		property	WindowParams:	TWindowParamsEx	read	FWindowParams;
		property	DisableCaption:	Boolean	read	FDisableCaption;
//		property	hwndProgman:	HWND	read	FhwndProgman;
		property	WindowFollowing:	Boolean	read	FWindowFollowing	write	SetWindowFollowing;
		property	MascotAnchor:	TAnchor	read	GetMascotAnchor;
		property	WindowAnchor:	TAnchor	read	GetWindowAnchor;
		property	XPos:	Integer	read	GetXPos;
		property	YPos:	Integer	read	GetYPos;
		property	Form2Rect:	TRect	read	FForm2Rect	write	FForm2Rect;
//		property	FolderMenuList:	TFolderMenuList	read	FFolderMenuList;
		property	TimeDifference:	Real	read	FTimeDifference	{write	SetTimeDifference};
		property	ModifiedNow:	TDateTime	read	GetModifiedNow;
		property	ModifiedDate:	TDateTime	read	GetModifiedDate;
		property	ModifiedTime:	TDateTime	read	GetModifiedTime;
{
		property	ACColor:	TColorRef	read	FACColor	write	FACColor;
		property	ACFont:	TLogFont	read	FACFont	write	FACFont;
		property	ACFontEnable:	Boolean	read	FACFontEnable	write	FACFontEnable;
		property	OrgACColor:	TColorRef	read	FOrgACColor	write	FOrgACColor;
		property	OrgACFont:	TLogFont	read	FOrgACFont	write	FOrgACFont;
		property	ACFontModified:	Boolean	read	FACFontModified	write	FACFontModified;
}
	published
		property	OnPositionChanged:	TPositionChangedEvent	read	FOnPositionChanged	write	FOnPositionChanged;
		property	OnWindowChanged:	TNotifyEvent	read	FOnWindowChanged	write	FOnWindowChanged;
		property	OnViewStyleChange:	TViewStyleChangeEvent	read	FOnViewStyleChange	write	FOnViewStyleChange;
	end;

	function	IsInternetConnected:	Boolean;
	function	GetAnchorPoint(const	ARect:TRect;Anchor:TAnchor):	TPoint;
	procedure	ParsePosition(const	ARect,BaseRect:TRect;var	Anchor,BaseAnchor:TAnchor);
	procedure	SplitMenu(AMenu:TMenuItem);

const
	SMonthNamesJ:	array[1..12]	of	string=(
		'ӷ',		'ǡ',		'',
		'',		'',		'̵',
		'ʸ',		'շ',		'Ĺ',
		'̵',	'',		''
	);

var
	DocForm1:	TDocForm1;
	LargeSysImages:	TImageList;
	SmallSysImages:	TImageList;

//	QrekConverter:	TQrekConverter;

implementation

uses
(*
	WinInet, ActiveX, ShellAPI,
*)
	IniFiles,
	MainFrm{, SysLVFrm};

resourcestring
	IniSectionMascot='Mascot';
	IniSectionWindow='Window';
	IniSectionPosition='Position';
	IniSectionPlugin='Plugin';
	IniSectionAlarm='Alarm';
	IniSectionTime='Time';
	IniSectionDesktop='Desktop';
	IniSectionFolderMenu='FolderMenu';
	IniSectionNTP='NTP';

//------------------------------------------------------------
//  Ѵؿ
//------------------------------------------------------------

function	IsMouseDown:	Boolean;
begin
{
	Result:=IsKeyDown(VK_LBUTTON)	or
					IsKeyDown(VK_RBUTTON)	or
					IsKeyDown(VK_MBUTTON);
}
	Result:=False;
end;

function	IsInternetConnected:	Boolean;
begin
	Result:=False;
end;

//ͭɥ˥⡼ܥåä饢ƥ֤ˤIEк
(*
procedure	ActivateModalDialog(hWindow:HWND);
type
	TEnumThreadWndParam=record
		hWindow:	HWND;
		hDlgbox:	HWND;
	end;
	PEnumThreadWndParam=^TEnumThreadWndParam;

	function	EnumThreadWndProc(hWindow:HWND;var	lParam:TEnumThreadWndParam):	LongBool;	stdcall;
	var
		hwndOwner:	HWND;
	begin
		if	IsModalDialog(hWindow)	then
		begin
			hwndOwner:=GetWindow(hWindow,GW_OWNER);
			if	(hwndOwner=lParam.hWindow)	and
					(GetWindowTextString(hWindow)<>'')	and
					(lParam.hDlgbox=HWND(nil))	then
				lParam.hDlgbox:=hWindow;
		end;

		Result:=True;
	end;

var
	Param:	TEnumThreadWndParam;
begin
	if	GetForegroundWindow<>hWindow	then	Exit;
	if	IsModalDialog(hWindow)	then  Exit;

	Param.hWindow:=hWindow;
	Param.hDlgbox:=HWND(nil);
	if	EnumThreadWindows(GetWindowThreadProcessId(hWindow,nil),@EnumThreadWndProc,LPARAM(@Param))	then
	begin
		if	(Param.hDlgbox<>HWND(nil))	and	IsWindowEnabled(Param.hDlgbox)	then
			SetForegroundWindow(Param.hDlgbox);
	end;
end;
*)

function	GetAnchorPoint(const	ARect:TRect;Anchor:TAnchor):	TPoint;

	//黻ҡѤ
	function	Alt(Cond:Boolean;TrueValue,FalseValue:Integer):	Integer;
	begin
		if	Cond	then
			Result:=TrueValue
		else
			Result:=FalseValue;
	end;

begin
	Result.x:=Alt(Anchor	in	[TopLeft,BottomLeft],ARect.Left,ARect.Right);
	Result.y:=Alt(Anchor	in	[TopLeft,TopRight],ARect.Top,ARect.Bottom);
end;

function	GetNearestAnchor(const	APoint,Origin:TPoint):	TAnchor;
begin
	Result:=TAnchor(Ord(APoint.x<Origin.x)+Ord(APoint.y<Origin.y)*2);
end;

function	GetCenterPt(ARect:TRect):	TPoint;
begin
	Result.x:=(ARect.Left+ARect.Right)	div	2;
	Result.y:=(ARect.Top+ARect.Bottom)	div	2;
end;

procedure	ParsePosition(const	ARect,BaseRect:TRect;var	Anchor,BaseAnchor:TAnchor);
var
	pt:	TPoint;
begin
	pt:=GetCenterPt(ARect);
	BaseAnchor:=GetNearestAnchor(GetCenterPt(BaseRect),pt);
	Anchor:=GetNearestAnchor(pt,GetAnchorPoint(BaseRect,BaseAnchor));
end;

procedure	SplitMenu(AMenu:TMenuItem);
begin
//do nothing.
end;

(*
//var
//	BreakSize:	Integer;
//	i:	Integer;
begin
{
	if	Assigned(AMenu.SubMenuImages)	and	(AMenu.SubMenuImages.Height>16)	then
		BreakSize:=10
	else
		BreakSize:=20;

	for	i:=(AMenu.Count-1)	div	BreakSize	downto	1	do
		AMenu[i*BreakSize].Break:=mbBarBreak;
}
end;
*)

//------------------------------------------------------------
//  TFolderMenuItem
//------------------------------------------------------------

(*
constructor	TFolderMenuItem.Create;
begin
	inherited;
end;

constructor	TFolderMenuItem.Create(Id:string;nFolder:Integer;Enabled:Boolean=False);
var
	pidl:	PItemIDList;
	sfi:	TSHFileInfo;
begin
	inherited	Create;

	FNumber:=nFolder;
	Self.Enabled:=Enabled;
	if	SHGetSpecialFolderLocation(Application.Handle,nFolder,pidl)=NOERROR	then
	begin
		FIdent:=Id;
		FItemIDList:=pidl;
		SHGetFileInfo(PChar(pidl),0,sfi,SizeOf(sfi),SHGFI_PIDL	or
			SHGFI_SYSICONINDEX	or	SHGFI_DISPLAYNAME);
		FImageIndex:=sfi.iIcon;
		FDisplayName:=sfi.szDisplayName;
	end;
end;

destructor	TFolderMenuItem.Destroy;
var
	Malloc:	IMalloc;
begin
	SHGetMalloc(Malloc);
	Malloc.Free(FItemIDList);

	inherited;
end;

{
begin
	Assert(Assigned(Mallocator));

	Mallocator.Free(FItemIDList);

	inherited;
end;
}

procedure	TFolderMenuItem.Assign(Item:TFolderMenuItem);
begin
	FIdent:=Item.Ident;
	FNumber:=Item.Number;
	FOpts:=Item.Opts;
	FImageIndex:=Item.ImageIndex;
	FDisplayName:=Item.DisplayName;
	FItemIDList:=CloneItemIDList(Item.ItemIDList);
end;

function	TFolderMenuItem.Clone:	TFolderMenuItem;
begin
	Result:=TFolderMenuItem.Create;
	Result.Assign(Self);
end;

function	TFolderMenuItem.GetEnabled:	Boolean;
begin
	Result:=fmoEnabled	in	FOpts;
end;

procedure	TFolderMenuItem.SetEnabled(Val:Boolean);
begin
	if	Val	then
		Include(FOpts,fmoEnabled)
	else
		Exclude(FOpts,fmoEnabled);
end;

function	TFolderMenuItem.GetSubMenu:	Boolean;
begin
	Result:=(fmoSubMenu	in	FOpts)	and	(not	(fmoRecursive	in	FOpts));
end;

procedure	TFolderMenuItem.SetSubMenu(Val:Boolean);
begin
	Exclude(FOpts,fmoRecursive);
	if	Val	then
		Include(FOpts,fmoSubMenu)
	else
		Exclude(FOpts,fmoSubMenu);
end;

function	TFolderMenuItem.GetSubMenuRecursive:	Boolean;
begin
	Result:=(fmoSubMenu	in	FOpts)	and	(fmoRecursive	in	FOpts);
end;

procedure	TFolderMenuItem.SetSubMenuRecursive(Val:Boolean);
begin
	if	Val	then
		FOpts:=FOpts+[fmoSubMenu,fmoRecursive]
	else
		FOpts:=FOpts-[fmoSubMenu,fmoRecursive];
end;

//------------------------------------------------------------
//  TFolderMenuList
//------------------------------------------------------------

constructor	TFolderMenuList.Create;
begin
	inherited;

	FList:=TObjectList.Create(True);
end;

destructor	TFolderMenuList.Destroy;
begin
	FList.Free;

	inherited;
end;

procedure	TFolderMenuList.Add(Item:TFolderMenuItem);
begin
	FList.Add(Item);
end;

procedure	TFolderMenuList.Add(S:string;nFolder:Integer;Enabled:Boolean);
var
	Item:	TFolderMenuItem;
begin
	Item:=TFolderMenuItem.Create(S,nFolder,Enabled);
	if	Assigned(Item.ItemIDList)	then
		Add(Item)
	else
		Item.Free;
end;

procedure	TFolderMenuList.Clear;
begin
	FList.Clear;
end;

procedure	TFolderMenuList.Assign(AList:TFolderMenuList);
var
	i:	Integer;
begin
	Clear;

	for	i:=0	to	AList.Count-1	do
		Add(AList[i].Clone);
end;

procedure	TFolderMenuList.Exchange(Index1,Index2:Integer);
begin
	FList.Exchange(Index1,Index2);
end;

function	TFolderMenuList.GetCount:	Integer;
begin
	Result:=FList.Count;
end;

function	TFolderMenuList.GetItems(Idx:Integer):	TFolderMenuItem;
begin
	Result:=FList[Idx]	as	TFolderMenuItem;
end;
*)

//------------------------------------------------------------
//  TCondList
//------------------------------------------------------------

procedure	TCondList.Assign(Source:TCondList);
var
	i:	Integer;
begin
	Clear;
	for	i:=0	to	Source.Count-1	do
		Add(Source[i]^);
end;

procedure TCondList.Clear;
var
	i:	Integer;
begin
	for	i:=Count-1	downto	0	do
		Delete(i);
end;

function TCondList.Get(Index:Integer):	PCondListItem;
begin
	Result:=PCondListItem(inherited	Get(Index));
end;

procedure TCondList.Put(Index:Integer;Item:PCondListItem);
var
	p:	PCondListItem;
begin
	if	Assigned(Item)	then
	begin
		if	Assigned(Items[Index])	then
			p:=Items[Index]
		else
		begin
			New(p);
			inherited	Put(Index,p);
		end;

		p^:=Item^;
	end
	else
		inherited	Put(Index,nil)
end;

function TCondList.Add(const	Item:TCondListItem): Integer;
var
	p:	PCondListItem;
begin
	New(p);
	p^:=Item;
	Result:=inherited	Add(p);
end;

procedure TCondList.Delete(Index: Integer);
begin
	Dispose(Items[Index]);
	Items[Index]:=nil;

	inherited	Delete(Index);
end;

function TCondList.IndexOf(Name:string): Integer;
begin
  Result := 0;
	while (Result<Count) and (Items[Result].Name<>Name) do
		Inc(Result);
	if Result=Count then
		Result:=-1;
end;

//------------------------------------------------------------
//  TWindowParamsEx
//------------------------------------------------------------

destructor	TWindowParamsEx.Destroy;
begin
	inherited;

	RecoverCaption;
end;

procedure	TWindowParamsEx.Change;
begin
	inherited;

	FOriginalWindowText:=inherited	GetWindowText;
	FModifiedWindowText:=FOriginalWindowText;
end;

procedure	TWindowParamsEx.UpdateTitle;
var
	Caption:	string;
begin
	Caption:=ModifiedWindowText;

	if	Caption<>FModifiedWindowText	then
	begin
		//Τʤ֤˥ȥ뤬ѹƤ
		FOriginalWindowText:=Caption;
		FModifiedWindowText:=Caption;
	end;
end;

procedure	TWindowParamsEx.SetHandle(Val:TWindow);
begin
	RecoverCaption;

	inherited;
end;

{
function	TWindowParamsEx.GetWindowClass:	string;
begin
	if	Assigned	then
		Result:=inherited	GetWindowClass
	else
		Result:='(Class)';
end;
}

function	TWindowParamsEx.GetWindowText:	string;
begin
	if	Assigned	then
	begin
		UpdateTitle;

		Result:=FOriginalWindowText;
	end
	else
		Result:='[Titile]';
end;

function	TWindowParamsEx.GetModuleName:	string;
begin
	Result:=inherited	GetModuleName;

	if	Result=''	then	Result:='(Unknown EXE Name)';
end;

function	TWindowParamsEx.GetModuleBaseName:	string;
begin
	Result:=inherited	GetModuleBaseName;

	if	Result=''	then	Result:='(Unknown EXE Name)';
end;

function	TWindowParamsEx.GetModifiedWindowText:	string;
begin
	Result:=inherited	GetWindowText;
end;

procedure	TWindowParamsEx.SetModifiedWindowText(Val:string);
begin
	inherited	SetWindowText(Val);

	FModifiedWindowText:=Val;
end;

procedure	TWindowParamsEx.RecoverCaption;
begin
	if	Assigned	then
	begin
//		if	FOriginalWindowText<>''	then
		UpdateTitle;
		WindowText:=FOriginalWindowText;

//		FOriginalWindowText:='';
	end;
end;

//------------------------------------------------------------
//  TDocForm1
//------------------------------------------------------------

constructor	TDocForm1.Create(View:TCustomForm);
//var
//	QTBL:	string;
begin
	inherited	Create(View);

	FMascotList:=TMascotList.Create;
	FCondList:=TCondList.Create;
	FWindowParams:=TWindowParamsEx.Create;
//	FFolderMenuList:=TFolderMenuList.Create;
//	FLex:= TLex.Create(WindowParams);
//	FYacc:= TYacc.Create(Lex);

	FMascotList.OnChange:=MascotListChange;

	FWindowFollowing:=True;
//	FhwndProgman:=FindWindow('Progman','Program Manager');

{
	QTBL:=ExtractFilePath(Application.ExeName)+'qtbl.txt';
	if	FileExists(QTBL)	then
		QrekConverter.LoadFromFile(QTBL);
}

	FLastTime:=Time;

	InitFolderMenuList;
end;

destructor	TDocForm1.Destroy;
begin
//	Yacc.Free;
//	Lex.Free;
//	FolderMenuList.Free;
	WindowParams.Free;
	CondList.Free;
	MascotList.Free;

	inherited;
end;

procedure	TDocForm1.Assign(Source:TPersistent);
begin
	if	Source	is	TDocForm1	then
	begin
		with	Source	as	TDocForm1	do
		begin
			Self.MascotList.Assign(MascotList);
			Self.CondList.Assign(CondList);
			Self.Field:=Field;
		end;
		Modified:=True;
	end	else
		inherited	Assign(Source);
end;

procedure	TDocForm1.BeginUpdate;
begin
	inherited;

	FTimerState:=Form1.Timer1.Enabled;
	Form1.Timer1.Enabled:=False;
end;

procedure	TDocForm1.EndUpdate;
begin
	Form1.Timer1.Enabled:=FTimerState;

	inherited;
end;

//------------------------------------------------------------

{	IniFile	}

procedure	TDocForm1.LoadIniFile;
var
	i:	Integer;
	n:	Integer;
	Name:	string;
	Picture:	string;
	Pos:	TPoint;
	Mascot:	TMascot;
	IniSection:	string;
//	EnablePluginDirs:	TEnablePluginDirs;
	NTPLastUpdateString:	string;
	CondListItem:	TCondListItem;
begin
	with	TMemIniFile.Create(IniFile)	do
	try
		n:=ReadInteger('Mascot','MascotCount',0);

		for	i:=0	to	n-1	do
		begin
			IniSection:=Format('Mascot%d',[i]);
			Name:=ReadString(IniSection,'Name','');
			Picture:=ReadString(IniSection,'Picture','');
			if	(Name='')	or	(Picture='')	then	Break;
			Mascot:=TMascot.Create(Name,Picture);
			with	Mascot	do
			begin
				Mask:=ReadString(IniSection,'Mask','');
				MaskType:=TMaskType(ord(ReadString(IniSection,'MaskType','A')[1])-ord('A'));
				MascotAnchor:=TAnchor(ReadInteger(IniSection,'MascotAnchor',Integer(BottomLeft)));
				WindowAnchor:=TAnchor(ReadInteger(IniSection,'WindowAnchor',Integer(BottomRight)));
				WindowFollowing:=ReadBool(IniSection,'WindowFollowing',True);
				XPos:=ReadInteger(IniSection,'XPos',0);
				YPos:=ReadInteger(IniSection,'YPos',0);
				Frequency:=ReadInteger(IniSection,'Frequency',100);
			end;
			MascotList.Add(Mascot);
		end;

		CurrentMascot:=ReadInteger(IniSectionMascot,'CurrentMascot',-1);
		ChangeInterval:=ReadInteger(IniSectionMascot,'ChangeInterval',0);
		InitialChange:=ReadBool(IniSectionMascot,'InitialChange',False);

		UseCaptionFmt:=ReadBool(IniSectionWindow,'UseCaptionFmt',False);
		CaptionFmt:=ReadString(IniSectionWindow,'CaptionFmt','');
		TimerInterval:=ReadInteger(IniSectionWindow,'TimerInterval',200);
		DefButtonEnable:=ReadBool(IniSectionWindow,'DefButtonEnable',False);
		SmartMove:=ReadBool(IniSectionWindow,'SmartMove',False);
		AllowDuplication:=ReadBool(IniSectionWindow,'AllowDuplication',False);
		ShowHint:=ReadBool(IniSectionWindow,'ShowHint',False);
		HintColor:=StringToColor(ReadString(IniSectionWindow,'HintColor',ColorToString(Application.HintColor)));
		UnionAsocWindows:=ReadBool(IniSectionWindow,'UnionAsocWindows',False);
		HideByFullScreenWindow:=ReadBool(IniSectionWindow,'HideByFullScreenWindow',False);
		EnableActivateModalDialog:=ReadBool(IniSectionWindow,'EnableActivateModalDialog',True);
		SensorIDE:=ReadBool(IniSectionWindow,'SensorIDE',False);

		if	Assigned(View)	then	Pos:=Point(View.Left,View.Top);
		Pos.x:=ReadInteger(IniSectionPosition,'Left',Pos.x);
		Pos.y:=ReadInteger(IniSectionPosition,'Top',Pos.y);
		PositionChanged(Pos.x,Pos.y);

		TopMost:=ReadBool(IniSectionPosition,'TopMost',True);
		AlwaysInScreen:=ReadBool(IniSectionPosition,'AlwaysInScreen',False);
		ZOrderControl:=ReadBool(IniSectionPosition,'ZOrderControl',True);

		DisableFollowings:=[dfDesktop,dfTopMost,dfMaximum,dfFarAway,dfRegion,dfTaskBar];
		n:=ReadInteger(IniSectionPosition,'DisableFollowings',Byte(DisableFollowings));
		DisableFollowings:=TDisableFollowings(Byte(n));

{
		UsePlugin:=ReadBool(IniSectionPlugin,'UsePlugin',False);
		n:=ReadInteger(IniSectionPlugin,'EnablePluginDirs',0);
//		EnablePluginDirs:=TEnablePluginDirs(Byte(n));
//		PluginDirs.CommaText:=ReadString(IniSectionPlugin,'PluginDirs','');
		PluginDirs:=ReadString(IniSectionPlugin,'PluginDirs','');

		if	PluginDirs=''	then
			PluginDirs:='$(Susie)'
		else
		if  UsePlugin and (pdSusie	in	EnablePluginDirs)  then
			PluginDirs:='$(Susie);'+PluginDirs;

		with	FField.BlinkerParams	do
		begin
			Kind:=TAlarmKind(ReadInteger(IniSectionAlarm,'BlinkerKind',ord(akNone)));
			if	Kind<>akNone	then
			begin
				if	Kind<>akPegion	then
					Count:=ReadInteger(IniSectionAlarm,'BlinkerCount',DefAlarmCount[Kind]);
				FreqLevel:=TFreqLevel(ReadInteger(IniSectionAlarm,'BlinkerFreqLevel',Ord(flNormal)));
				if	FreqLevel=flCustom	then
					Freq:=ReadInteger(IniSectionAlarm,'BlinkerFreq',BlinkerFreqs[flNormal]);
				HalfHourAlarm:=ReadBool(IniSectionAlarm,'BlinkerHalfHourAlarm',False);
			end;
		end;

		with	FField.ShakerParams	do
		begin
			Kind:=TAlarmKind(ReadInteger(IniSectionAlarm,'ShakerKind',ord(akNone)));
			if	Kind<>akNone	then
			begin
				if	Kind<>akPegion	then
					Count:=ReadInteger(IniSectionAlarm,'ShakerCount',DefAlarmCount[Kind]);
				FreqLevel:=TFreqLevel(ReadInteger(IniSectionAlarm,'ShakerFreqLevel',Ord(flNormal)));
				if	FreqLevel=flCustom	then
					Freq:=ReadInteger(IniSectionAlarm,'ShakerFreq',ShakerFreqs[flNormal]);
				HalfHourAlarm:=ReadBool(IniSectionAlarm,'ShakerHalfHourAlarm',False);
			end;
		end;

		with	FField.PlayerParams	do
		begin
			Kind:=TAlarmKind(ReadInteger(IniSectionAlarm,'PlayerKind',ord(akNone)));
			if	Kind<>akNone	then
			begin
				Count:=ReadInteger(IniSectionAlarm,'PlayerCount',DefAlarmCount[Kind]);
				FreqLevel:=TFreqLevel(ReadInteger(IniSectionAlarm,'PlayerFreqLevel',Ord(flNormal)));
				if	FreqLevel=flCustom	then
					Freq:=ReadInteger(IniSectionAlarm,'PlayerFreq',ShakerFreqs[flNormal]);
				HalfHourAlarm:=ReadBool(IniSectionAlarm,'PlayerHalfHourAlarm',False);
				FileName:=ReadString(IniSectionAlarm,'SoundFile','');
			end;
		end;
}

		TimeOffset:=ReadInteger(IniSectionTime,'TimeOffset',0);

{
		UseDesktop:=ReadBool(IniSectionDesktop,'UseDesktop',False);
}
		IconState:=TIconState(ReadInteger(IniSectionDesktop,'IconState',ord(isNone)));

{
		for	i:=0	to	FolderMenuList.Count-1	do
			with	FolderMenuList[i]	do
			begin
				n:=ReadInteger(IniSectionFolderMenu,Ident,Byte(Opts));
				Opts:=TFolderMenuOpts(Byte(n));
			end;
}

		NTPServer:=ReadString(IniSectionNTP,'NTPServer','');
		NTPTimeout:=ReadInteger(IniSectionNTP,'NTPTimeout',10);
		NTPInterval:=TNTPInterval(ReadInteger(IniSectionNTP,'NTPInterval',Byte(ntpiPerDay)));
		NTPLastUpdateString:=ReadString(IniSectionNTP,'NTPLastUpdate','');
		NTPSyncPeriodic:=ReadBool(IniSectionNTP,'NTPSyncPeriodic',False);
		NTPSyncSystem:=ReadBool(IniSectionNTP,'NTPSyncSystem',False);
		NTPSyncGradually:=ReadBool(IniSectionNTP,'NTPSyncGradually',False);
		NTPOnlyIfConnected:=ReadBool(IniSectionNTP,'NTPOnlyIfConnected',True);
		NTPServerValid:=(NTPServer<>'');

		if	NTPLastUpdateString<>''	then
		try
			NTPLastUpdate:=StrToDateTime(NTPLastUpdateString);
		except
			on	EConvertError	do
				NTPLastUpdate:=0;
		end;

		n:=ReadInteger('Cond','CondCount',0);
		for	i:=0	to	n-1	do
		begin
			IniSection:=Format('Cond%d',[i]);

			with	CondListItem	do
			begin
				Name:=ReadString(IniSection,'Name',IniSection);
				WindowClass:=ReadString(IniSection,'WindowClass','');
				WindowText:=ReadString(IniSection,'WindowText','');
				ExeName:=ReadString(IniSection,'ExeName','');
				Width:=ReadInteger(IniSection,'Width',0);
				Height:=ReadInteger(IniSection,'Height',0);
				Flags:=TCondFlags(Byte(ReadInteger(IniSection,'Flags',0)));
				Following:=ReadBool(IniSection,'Following',WindowFollowing);
				DefButton:=ReadBool(IniSection,'DefButton',DefButtonEnable);
				CaptionFmt:=ReadBool(IniSection,'CaptionFmt',UseCaptionFmt);
				AsocWindows:=ReadBool(IniSection,'AsocWindows',UnionAsocWindows);
				Dialog:=ReadBool(IniSection,'Dialog',EnableActivateModalDialog);
			end;
			CondList.Add(CondListItem);
		end;

		if	CondList.Count=0	then
		begin
			with	CondListItem	do
			begin
				Name:='(ǥեȤ)';
				WindowClass:='';
				WindowText:='';
				ExeName:='';
				Width:=0;
				Height:=0;
				Flags:=[cfFollowing,cfDefButton,cfCaption,cfAsocWindows,cfDialog];
				Following:=WindowFollowing;
				DefButton:=DefButtonEnable;
				CaptionFmt:=UseCaptionFmt;
				AsocWindows:=UnionAsocWindows;
				Dialog:=EnableActivateModalDialog;
			end;
			CondList.Add(CondListItem);
		end;

		Form2Rect:=Bounds(
			ReadInteger(IniSectionDesktop,'DesktopLeft',0),
			ReadInteger(IniSectionDesktop,'DesktopTop',0),
			ReadInteger(IniSectionDesktop,'DesktopWidth',0),
			ReadInteger(IniSectionDesktop,'DesktopHeight',0)
		);
	finally
		Free;
	end;
	Modified:=False;
end;

procedure	EraseSectionSeries(IniFile:TCustomIniFile;Prefix:string;Start:Integer=0);
var
	n:	Integer;
	IniSection:	string;
begin
	n:=Start;
	while	True	do
	begin
		IniSection:=Format('%s%d',[Prefix,n]);
		if	not	IniFile.SectionExists(IniSection)	then	Break;
		IniFile.EraseSection(IniSection);
		Inc(n);
	end;
end;

procedure	TDocForm1.SaveIniFile;
var
	i:	Integer;
	IniSection:	string;
	MemIni:	TMemIniFile;
begin
	if	ReadOnly	then	Exit;

	BeginUpdate;
	try
		MemIni:=TMemIniFile.Create(IniFile);
		with	MemIni	do
		try
//			n:=MascotList.Count;
			EraseSectionSeries(MemIni,'Mascot');

			for	i:=0	to	MascotList.Count-1	do
			begin
				IniSection:=Format('Mascot%d',[i]);
				with	MascotList[i]	do
				begin
					WriteString(IniSection,'Name',Name);
					WriteString(IniSection,'Picture',Picture);
					WriteString(IniSection,'MaskType',Char(ord(MaskType)+ord('A')));
					if	Mask<>''	then
						WriteString(IniSection,'Mask',Mask);
					WriteInteger(IniSection,'MascotAnchor',Integer(MascotAnchor));
					WriteInteger(IniSection,'WindowAnchor',Integer(WindowAnchor));
					WriteBool(IniSection,'WindowFollowing',WindowFollowing);
					WriteInteger(IniSection,'XPos',XPos);
					WriteInteger(IniSection,'YPos',YPos);
					WriteInteger(IniSection,'Frequency',Frequency);
				end;
			end;

			WriteInteger(IniSectionMascot,'MascotCount',MascotList.Count);
			WriteInteger(IniSectionMascot,'CurrentMascot',CurrentMascot);
			WriteInteger(IniSectionMascot,'ChangeInterval',ChangeInterval);
			WriteBool(IniSectionMascot,'InitialChange',InitialChange);

			WriteBool(IniSectionWindow,'UseCaptionFmt',UseCaptionFmt);
			WriteString(IniSectionWindow,'CaptionFmt',CaptionFmt);
			WriteInteger(IniSectionWindow,'TimerInterval',TimerInterval);
			WriteBool(IniSectionWindow,'DefButtonEnable',DefButtonEnable);
			WriteBool(IniSectionWindow,'SmartMove',SmartMove);
			WriteBool(IniSectionWindow,'AllowDuplication',AllowDuplication);
			WriteBool(IniSectionWindow,'ShowHint',ShowHint);
			WriteString(IniSectionWindow,'HintColor',ColorToString(HintColor));
			WriteBool(IniSectionWindow,'UnionAsocWindows',UnionAsocWindows);
			WriteBool(IniSectionWindow,'HideByFullScreenWindow',HideByFullScreenWindow);
			WriteBool(IniSectionWindow,'EnableActivateModalDialog',EnableActivateModalDialog);
			WriteBool(IniSectionWindow,'SensorIDE',SensorIDE);

//			Application.ProcessMessages;

			if	Assigned(View)	then
			begin
				WriteInteger(IniSectionPosition,'Left',View.Left);
				WriteInteger(IniSectionPosition,'Top',View.Top);
			end;

			WriteBool(IniSectionPosition,'TopMost',TopMost);
			WriteBool(IniSectionPosition,'AlwaysInScreen',AlwaysInScreen);
			WriteBool(IniSectionPosition,'ZOrderControl',ZOrderControl);

			WriteInteger(IniSectionPosition,'DisableFollowings',Byte(DisableFollowings));

//			Application.ProcessMessages;

			EraseSection(IniSectionPlugin);
			WriteBool(IniSectionPlugin,'UsePlugin',UsePlugin);
//			if	PluginDirs=''	then
//				EnablePluginDirs:=EnablePluginDirs-[pdExtra];
//			WriteInteger(IniSectionPlugin,'EnablePluginDirs',Byte(EnablePluginDirs));
			if	PluginDirs<>''	then
				WriteString(IniSectionPlugin,'PluginDirs',PluginDirs);

//			Application.ProcessMessages;

{
			with	BlinkerParams	do
			begin
				WriteInteger(IniSectionAlarm,'BlinkerKind',Ord(Kind));
				if	Kind<>akNone	then
				begin
					WriteInteger(IniSectionAlarm,'BlinkerCount',Count);
					WriteInteger(IniSectionAlarm,'BlinkerFreqLevel',Ord(FreqLevel));
					if	FreqLevel=flCustom	then
						WriteInteger(IniSectionAlarm,'BlinkerFreq',Freq);
					WriteBool(IniSectionAlarm,'BlinkerHalfHourAlarm',HalfHourAlarm);
				end;
			end;

			with	ShakerParams	do
			begin
				WriteInteger(IniSectionAlarm,'ShakerKind',ord(Kind));
				if	Kind<>akNone	then
				begin
					WriteInteger(IniSectionAlarm,'ShakerCount',Count);
					WriteInteger(IniSectionAlarm,'ShakerFreqLevel',Ord(FreqLevel));
					if	FreqLevel=flCustom	then
						WriteInteger(IniSectionAlarm,'ShakerFreq',Freq);
					WriteBool(IniSectionAlarm,'ShakerHalfHourAlarm',HalfHourAlarm);
				end;
			end;

			with	PlayerParams	do
			begin
				WriteInteger(IniSectionAlarm,'PlayerKind',ord(Kind));
				if	Kind<>akNone	then
				begin
					WriteInteger(IniSectionAlarm,'PlayerCount',Count);
					WriteInteger(IniSectionAlarm,'PlayerFreqLevel',Ord(FreqLevel));
					if	FreqLevel=flCustom	then
						WriteInteger(IniSectionAlarm,'PlayerFreq',Freq);
					WriteBool(IniSectionAlarm,'PlayerHalfHourAlarm',HalfHourAlarm);
					WriteString(IniSectionAlarm,'SoundFile',FileName);
				end;
			end;
//			Application.ProcessMessages;
}

			WriteInteger(IniSectionTime,'TimeOffset',TimeOffset);

			WriteBool(IniSectionDesktop,'UseDesktop',UseDesktop);
			if	UseDesktop	then
			begin
				WriteInteger(IniSectionDesktop,'IconState',ord(IconState));
{
				with	Form2	do
				begin
					WriteInteger(IniSectionDesktop,'DesktopLeft',Left);
					WriteInteger(IniSectionDesktop,'DesktopTop',Top);
					WriteInteger(IniSectionDesktop,'DesktopWidth',Width);
					WriteInteger(IniSectionDesktop,'DesktopHeight',Height);
				end;

				EraseSection(IniSectionFolderMenu);
				for	i:=0	to	FolderMenuList.Count-1	do
				begin
					with	FolderMenuList[i]	do
						WriteInteger(IniSectionFolderMenu,Ident,Byte(Opts));
				end;
}
			end;

			WriteString(IniSectionNTP,'NTPServer',NTPServer);
			WriteInteger(IniSectionNTP,'NTPTimeout',NTPTimeout);
			WriteInteger(IniSectionNTP,'NTPInterval',Byte(NTPInterval));
			if	NTPLastUpdate=0	then
				WriteString(IniSectionNTP,'NTPLastUpdate','')
			else
				WriteString(IniSectionNTP,'NTPLastUpdate',DateTimeToStr(NTPLastUpdate));
			WriteBool(IniSectionNTP,'NTPSyncPeriodic',NTPSyncPeriodic);
			WriteBool(IniSectionNTP,'NTPSyncSystem',NTPSyncSystem);
			WriteBool(IniSectionNTP,'NTPSyncGradually',NTPSyncGradually);
			WriteBool(IniSectionNTP,'NTPOnlyIfConnected',NTPOnlyIfConnected);

			EraseSectionSeries(MemIni,'Cond');
			for	i:=0	to	CondList.Count-1	do
			begin
				IniSection:=Format('Cond%d',[i]);

				with	CondList[i]^	do
				begin
					WriteString(IniSection,'Name',Name);
					WriteString(IniSection,'WindowClass',WindowClass);
					WriteString(IniSection,'WindowText',WindowText);
					WriteString(IniSection,'ExeName',ExeName);
					WriteInteger(IniSection,'Width',Width);
					WriteInteger(IniSection,'Height',Height);
					WriteInteger(IniSection,'Flags',Byte(Flags));
					WriteBool(IniSection,'Following',Following);
					WriteBool(IniSection,'DefButton',DefButton);
					WriteBool(IniSection,'CaptionFmt',CaptionFmt);
					WriteBool(IniSection,'AsocWindows',AsocWindows);
					WriteBool(IniSection,'Dialog',Dialog);
				end;
			end;
			WriteInteger('Cond','CondCount',CondList.Count);

			UpdateFile;
		finally
			Free;
		end;
	finally
		EndUpdate;
	end;
	Modified:=False;
end;

//------------------------------------------------------------

{	Event	}

procedure	TDocForm1.PositionChanged(x,y:Integer);
begin
	if	Assigned(OnPositionChanged)	then
		OnPositionChanged(Self,x,y);
end;

procedure	TDocForm1.ViewStyleChange(ViewStyle:TViewStyle);
begin
	if	Assigned(OnViewStyleChange)	then
		OnViewStyleChange(Self,ViewStyle);
end;

procedure	TDocForm1.MascotListChange(Sender:TObject);
begin
	Modified:=True;
end;

(*
function	TDocForm1.SetTimeDifference(Val:Real):	Boolean;

	function	IgnoreTimeDifference(Val:Real;MinToWarn:Integer=3):	Boolean;
	const
		Fmt='%sʾפäƤޤ'#13+
				':%d'#13+
				'ͥåȥΤǤʤǽޤ'#13+
				'ĴƤǤ';
	var
		Msg:	string;
		AbsVal:	Real;
	begin
		AbsVal:=Abs(Val);

		if	AbsVal>60*60*24*365	then
			Msg:='ǯ'
		else
		if	AbsVal>60*60*24	then
			Msg:=''
		else
		if	AbsVal>60*60	then
			Msg:=''
		else
		if	(MinToWarn>0)	and	(AbsVal>60*MinToWarn)	then
			Msg:=Format('%dʬʾ',[MinToWarn]);

		Result:=False;
		if	(Msg='')	or
				(Application.MessageBox(
					Format(Fmt,[Msg,Round(Val)]),
					Application.Title,
					[smbYes,smbNo],smsWarning
				)=smbYes)	then	Exit;

		Result:=True;
	end;

begin
	NTPLastUpdate:=Now;

	if	Val=0	then
	begin
		FTimeDifference:=0;
		raise	ESNTPThreadError.Create('ƥλפΤǤ');
	end;

	if	NTPSyncGradually	then	Val:=Val/2;

	if	IgnoreTimeDifference(Val)	then
		raise	ESNTPThreadError.Create('Ʊߤޤ');

	FTimeDifference:=Val;

	Result:=NTPSyncSystem;
	if	Result	then
		SyncClock(TimeDifference);
end;
*)

//------------------------------------------------------------

{	WindowCaption	}

procedure	TDocForm1.UpdateCaption;
begin
	if	CaptionFmt<>''	then
	begin
{
		Lex.TimeOffset:=TimeOffset;
		Lex.TimeDifference:=TimeDifference;
		Lex.InputString:=CaptionFmt+#0;
		Lex.OutputStream:=nil;
		Yacc.yyparse;
		WindowParams.ModifiedWindowText:=Lex.OutputString;
}
	end;
end;

procedure	TDocForm1.UpdateDisableCaption;

{
	function	EnumThreadWndProc(hWindow:HWND;var	DisableCaption:Boolean):	LongBool;	stdcall;
	begin
		Result:=True;
		if	GetClassNameString(hWindow)='TPropertyInspector'	then
		begin
			DisableCaption:=True;
			Result:=False;
		end;
	end;
}

begin
	FDisableCaption:=True;

{
	if	(not	UseCaptionFmt)	or
			(not IsCondOk(WindowParams,cfCaption))	then	Exit;

	//ȥСʤɥϡץ񤭴оݤȤʤ
	if	(WindowParams.Style	and	WS_CAPTION)=0	then	Exit;

	//ѷ꡼󥦥ɥϡץ񤭴оݤȤʤ
	if	IsWindowRgnTransformed(WindowParams.Handle)	then	Exit;

	//ȥ뤬ʸΥɥϡץ񤭴оݤȤʤ
	if	WindowParams.WindowText=''	then	Exit;

	FDisableCaption:=False;

	//߷FormΥץ񤭴ʤθ
	if	SensorIDE	and	(WindowParams.WindowClass<>DialogClass)	then
		EnumThreadWindows(GetWindowThreadProcessId(WindowParams.Handle,nil),@EnumThreadWndProc,LPARAM(@FDisableCaption));
}
end;

//------------------------------------------------------------

{	CursorPos	}

procedure	TDocForm1.SaveCursorPos(pt:TPoint);
begin
	FCursorPos:=pt;
	IsCursorPosStored:=True;
end;

procedure	TDocForm1.RecoverCursorPos;
begin
//	if	not	IsCursorPosStored	then	Exit;

	ValidateCursorPos;

	if	not	IsCursorPosStored	then	Exit;
	IsCursorPosStored:=False;

	if	IsMouseDown	then	Exit;
	Mouse.CursorPos:=CursorPos;
//	SetCursorPos(CursorPos.x,CursorPos.y);
end;

{
var
	Pt:	TPoint;
begin
	if	IsCursorPosStored	then
	begin
		GetCursorPos(Pt);
		if	PtInRect(WindowParams.WindowRect,Pt)	and
				(not	IsMouseDown)	then
			SetCursorPos(CursorPos.x,CursorPos.y);
		IsCursorPosStored:=False;
	end;
end;
}

procedure	TDocForm1.ValidateCursorPos;
var
	Pt:	TPoint;
//	hWindow:	HWND;
begin
	if	not	Field.SmartMove	then	Exit;

	if	IsCursorPosStored	then
	begin
		GetCursorPos(Pt);

{
		if	WindowParams.IsWindow	and	WindowParams.IsVisible	then
		begin
			//ɥ¸ߤĥޥ¾Υɥξˤ
			hWindow:=WindowFromPoint(Pt);
			if	(	(WindowParams.Handle<>hWindow)	and
						(not	WindowParams.IsChild(hWindow))	)	then
				IsCursorPosStored:=False;
		end
		else	begin
			//ɥ¸ߤĥޥɥΤäγ˽ФƤ
			if	(not	IsRectEmpty(WindowParams.WindowRect))	and
					(not	PtInRect(WindowParams.WindowRect,Pt))	then
				IsCursorPosStored:=False;
		end;
}
	end;
end;

procedure	MoveCursor(X,Y:Integer);
const
	d=10;
var
	pt:	TPoint;
	dx,dy:	Integer;
	r:	Real;
	px,py:	Integer;
//	MouseParams:	array[0..2]	of	Integer;
begin
//	SystemParametersInfo(SPI_MOUSE,0,@MouseParams,0);

	while	True	do
	begin
		GetCursorPos(pt);
		dx:=X-pt.x;	dy:=Y-pt.y;
		r:=Sqrt(dx*dx+dy*dy);
		if	r<1	then	Break;
		if	r>d	then
		begin
			px:=Trunc(pt.x+dx*d/r);
			py:=Trunc(pt.y+dy*d/r);
		end
		else	begin
			px:=X;
			py:=Y;
		end;
		Mouse.CursorPos:=Point(px,py);
//		SetCursorPos(px,py);
	end;
end;

procedure	TDocForm1.SnapToDefButton;
begin
end;

(*
var
//	hwndControl:	HWND;
//	WndRect:	TRect;
	pt:	TPoint;
//	dx,dy:	Integer;
//	i:	Integer;
begin
	if	DefButtonDone	then	Exit;

	//ɽξ
	if	not	WindowParams.IsVisible	then	Exit;

	DefButtonDone:=True;

	//ɥǤϤʤ
	if	not	IsDialog(WindowParams.Handle)	then	Exit;

	//ݥåץåץ˥塼줿ʥȥ쥤к
	if	IsMenuPopedup	then	Exit;

	hwndControl:=GetDefButton(WindowParams.Handle);

	if	(hwndControl<>HWND(nil))	and
			(not	IsMouseDown)	then
	begin
		GetWindowRect(hwndControl,wndRect);
		GetCursorPos(pt);

		//˥ǥեȥܥξ˥ޥаưʤ
		if	PtInRect(wndRect,pt)	then	Exit;

		SaveCursorPos(pt);

		with	wndRect	do
		begin
			//ϢкǾ餹˿VN:-P
			MoveCursor(Left-4,(Top+Bottom)	div	2);
{
			dx:=Left-4-pt.x;
			dy:=(Top+Bottom)	div	2-pt.y;
			for	i:=1	to	40	do
				SetCursorPos(pt.x+dx*i	div	40,pt.y+dy*i	div	40);
}
		end;
	end;
end;
*)

//------------------------------------------------------------

{	Alarm / SNTP	}

(*
procedure	TDocForm1.ThreadDone(Sender:TObject);
begin
{
	if	Sender=FBlinkerThread	then
		FBlinkerThread:=nil
	else
	if	Sender=FShakerThread	then
		FShakerThread:=nil
	else
	if	Sender=FPlayerThread	then
		FPlayerThread:=nil
	else
}
	if	Sender=FNTPThread	then
	begin
		if	FNTPThread.Succeeded	then
			SetTimeDifference(FNTPThread.TimeDifference)
		else
			Application.MessageBox(
				PChar(FNTPThread.LastErrorMsg),
				'SNTP Error',
				[smbOk],smsCritical
			);
		FNTPThread:=nil;
	end
	else
		Assert(False);
end;
*)

procedure	TDocForm1.TimeChanged;

	function	IsAlarm(Min,LMin,AMin:Integer):	Boolean;
	begin
		Result:=(Min=AMin)	and	(LMin<>AMin);
//		Result:=(Min=AMin)	and	(((Min-LMin+60)	mod	60)=1);
	end;


{
	function	AlarmEnable(var	Params:TAlarmParams;Hour,Min,LHour,LMin:Integer):	Boolean;
	begin
		Result:=False;

		with	Params	do
		begin
			if	Kind=akNone	then	Exit;

			if	((Hour-LHour+24)	mod	24=1)	and	IsAlarm(Min,LMin,0)	then
			begin
				Result:=True;
				if	Kind=akPegion	then	Count:=((Hour+11)	mod	12)+1;
			end
			else
			if	HalfHourAlarm	and
				(Hour=LHour)	and	IsAlarm(Min,LMin,30)	then
			begin
				Result:=True;
				if	Kind=akPegion	then	Count:=1;
			end;
		end;
	end;

	procedure	RunThread(var	AThreadRef:TAlarmThread;AThreadClass:TAlarmThreadClass;const	Params:TAlarmParams);
	begin
		if	Assigned(AThreadRef)	then	Exit;

		AThreadRef:=AThreadClass.Create(Params,ThreadDone);
	end;

	function  NTPIntervalToSec(NTPInterval:TNTPInterval):	Real;
	begin
		case	NTPInterval	of
			ntpiPerHour:	Result:=1/24;
			ntpiPerDay:		Result:=1;
			ntpiPerWeek:	Result:=7;
			ntpiPerMonth:	Result:=30;
			else					Result:=High(Integer);
		end;
	end;

	function	NTPEnable:	Boolean;
	begin
		Result:=False;

		if	Assigned(FNTPThread)	then	Exit;
		if	NTPServerValid=False	then	Exit;

		if	(NTPLastUpdate=0)	or	(Now-NTPLastUpdate>NTPIntervalToSec(NTPInterval))	then
		begin
			if	NTPOnlyIfConnected	and	(not	IsInternetConnected)	then	Exit;

			Result:=True;
		end;
	end;
}

var
	tm:	TDateTime;
	Hour:	Word;
	Min:	Word;
	LHour:	Word;
	LMin:	Word;
	Dummy:	Word;
//	Params:	TAlarmParams;
//	HostName:	string;
//	Port:	Integer;
begin
	tm:=ModifiedTime;
	DecodeTime(FLastTime,LHour,LMin,Dummy,Dummy);
	DecodeTime(tm,Hour,Min,Dummy,Dummy);
	FLastTime:=tm;

{
	Params:=BlinkerParams;
	if	AlarmEnable(Params,Hour,Min,LHour,LMin)	then
		RunThread(FBlinkerThread,TBlinkerThread,Params);

	Params:=ShakerParams;
	if	(not	IsFullScreenWindow(WindowParams.Handle))	and
			AlarmEnable(Params,Hour,Min,LHour,LMin)	then
		RunThread(FShakerThread,TShakerThread,Params);

	Params:=PlayerParams;
	if	AlarmEnable(Params,Hour,Min,LHour,LMin)	then
		RunThread(FPlayerThread,TPlayerThread,Params);

	if	NTPSyncPeriodic	and	NTPEnable	then
	begin
		NTPLastUpdate:=Now;
		NTPServerValid:=GetHostName(NTPServer,HostName,Port);
		if	NTPServerValid=False	then
		begin
			ShowMessage('ۥȤλְ꤬äƤޤ');
			Exit;
		end;

		FNTPThread:=TSNTPThread.Create(HostName,Port);
		FNTPThread.OnTerminate:=ThreadDone;
		FNTPThread.TimeOut:=NTPTimeout;

		FNTPThread.Resume;
	end;
}
end;

//------------------------------------------------------------

{
function	TDocForm1.IsCondOk(WindowParams:TWindowParams;Query:TCondFlag):	Boolean;

	function	CompareStrSubstr(BaseStr,SubStr:string):	Boolean;
	begin
		Result:=Pos(SubStr,BaseStr)>0;
	end;

	function	CompareTextSubstr(BaseStr,SubStr:string):	Boolean;
	begin
		Result:=Pos(LowerCase(SubStr),LowerCase(BaseStr))>0;
	end;

	function	MatchConditions(WindowParams:TWindowParams;CondListItem:TCondListItem):	Boolean;
	begin
		Result:=True;

		if	(CondListItem.WindowClass<>'')	and	CompareStrSubstr(WindowParams.WindowClass,CondListItem.WindowClass)	then	Exit;
		if	(CondListItem.WindowText<>'')		and	CompareStrSubstr(WindowParams.WindowText,CondListItem.WindowText)		then	Exit;
		if	(CondListItem.ExeName<>'')			and	CompareTextSubstr(WindowParams.ModuleName,CondListItem.ExeName)				then	Exit;

		if	(CondListItem.Width>0)	and	(CondListItem.Width>WindowParams.WindowWidth)			then	Exit;
		if	(CondListItem.Width<0)	and	(-CondListItem.Width<WindowParams.WindowWidth)		then	Exit;
		if	(CondListItem.Height>0)	and	(CondListItem.Height>WindowParams.WindowHeight)		then	Exit;
		if	(CondListItem.Height<0)	and	(-CondListItem.Height<WindowParams.WindowHeight)	then	Exit;

		Result:=False;
	end;

	function	CondToFlags(Cond:TCondListItem):	TCondFlags;
	begin
		Result:=[];
		if	Cond.Following		then	Include(Result,cfFollowing);
		if	Cond.DefButton		then	Include(Result,cfDefButton);
		if	Cond.CaptionFmt		then	Include(Result,cfCaption);
		if	Cond.AsocWindows	then	Include(Result,cfAsocWindows);
		if	Cond.Dialog				then	Include(Result,cfDialog);
	end;

	function	IsEnabled(Cond:TCondListItem;Flag:TCondFlag):	Boolean;
	begin
		Result:=Flag	in	CondToFlags(Cond);
	end;

var
	i:	Integer;
begin
	Result:=IsEnabled(CondList[0]^,Query);

	for	i:=1	to	CondList.Count-1	do
		if	MatchConditions(WindowParams,CondList[i]^)	and
				(Query	in	CondList[i].Flags)	then
			Result:=IsEnabled(CondList[i]^,Query);
end;
}

function	TDocForm1.AllowFollowing:	Boolean;
//var
//	R:	TRect;
begin
	Result:=False;

	//ʬȤƥ֤ˤʤä(ƥ֤ʥɥʤʤä)
	if	Assigned(View)	and	(WindowParams.Handle=QWidget_winId(View.Handle))	then	Exit;

	//ԲĻ륦ɥƥ
	if	not	WindowParams.IsVisible	then	Exit;

(*
	//Desktopɥɤ
	if	(dfDesktop	in	DisableFollowings)	and
		(WindowParams.Handle=hwndProgman)	then	Exit;

	//Сɤ
	if	(dfTaskBar	in	DisableFollowings)	and
		(WindowParams.WindowClass='Shell_TrayWnd')	then	Exit;

	//TopMostʥɥɤ
	if	(dfTopMost	in	DisableFollowings)	and
		((WindowParams.ExStyle	and	WS_EX_TOPMOST)<>0)	then	Exit;

	//粽ɥɤ
	if	(dfMaximum	in	DisableFollowings)	and
		WindowParams.IsMaximized	then	Exit;

	//̳ɽ줿ɥɤ
	if	(dfFarAway	in	DisableFollowings)	then
	begin
		IntersectRect(R,WindowParams.WindowRect,GetDeskTopRect);
		if	IsRectEmpty(R)	then	Exit;
	end;

	//ѷ꡼󥦥ɥɤ
	if	(dfRegion	in	DisableFollowings)	and
		IsWindowRgnTransformed(WindowParams.Handle)	then	Exit;

	if 	not IsCondOk(WindowParams,cfFollowing)	then	Exit;
*)

	Result:=True;
end;

procedure	TDocForm1.Timer1Timer;
var
	Win:	TWindow;
begin
	Win:=GetActiveWindow;

	if	WindowParams.Handle<>Win	then
	begin
		if  ZOrderControl	and
				(Win=QWidget_winId(View.Handle))	and
				(WindowParams.Handle<>TWindow(nil))	and
				(not	WindowParams.IsVisible)	then
		begin
			//ƥ֥ɥä˼ʬƥ֤ˤʤä
			Win:=GetNextVisibleWindow(QWidget_winId(Application.AppWidget));
			if	win<>TWindow(nil)	then
			begin
				SetActiveWindow(Win);
//				Exit;
			end;

//			SetWindowPos(hWindow,HWND_BOTTOM,0,0,0,0,SWP_NOMOVE	or	SWP_NOSIZE	or	SWP_HIDEWINDOW);
		end;

		RecoverCursorPos;
		DefButtonDone:=False;

		WindowParams.Handle:=win;
		if	Assigned(OnWindowChanged)	then
			OnWindowChanged(Self);

		UpdateDisableCaption;
	end;

{
	if	DefButtonEnable	and
			IsCondOk(WindowParams,cfDefButton)	then
	begin
		ValidateCursorPos;
		if	not	DefButtonDone	then
			SnapToDefButton;
	end;
}

	if	not	DisableCaption	then
		UpdateCaption;
end;

procedure	TDocForm1.Timer2Timer;
begin
{
	if	EnableActivateModalDialog	and
			IsCondOk(WindowParams,cfDialog)	then
		ActivateModalDialog(WindowParams.Handle);

	if	HideByFullScreenWindow	and
			(WindowParams.Handle<>hwndProgman)	and
			IsFullScreenWindow(WindowParams.Handle)	then
	begin
		ViewStyleChange(vsHide);
	end;
}
	TimeChanged;
end;

{
function	IsWindowStable(hWindow:HWND):	Boolean;
begin
	Result:=IsWindowVisible(hWindow)	and
//					(not	IsPopupMenu(hWindow))	and
					(not	IsToolTips(hWindow))	and
					(hWindow<>DocForm1.hwndProgman)	and
					(hWindow<>Form1.Handle);
end;
}

{
function	TDocForm1.GetAsocWindowsRect(hWindow:HWND):	TRect;

	function	EnumThreadWndProc(hWindow:HWND;lParam:PRect):	LongBool;	stdcall;
	var
		Rect:	TRect;
	begin
		Result:=True;
		if	IsWindowStable(hWindow)	then
		begin
			Windows.GetWindowRect(hWindow,Rect);
			UnionRect(lParam^,lParam^,Rect);
		end;
	end;

var
	dwThreadID:	DWORD;
begin
	Result:=Rect(0,0,0,0);

	if	not	IsWindow(hWindow)	then	Exit;

	dwThreadID:=GetWindowThreadProcessId(hWindow,nil);
	EnumThreadWindows(dwThreadID,@EnumThreadWndProc,LPARAM(@Result));
end;
}

procedure	TDocForm1.UpdatePosition(var	pt:TPoint;const	MascotRect:TRect);

	function	AdjustValue(Min,Max:Integer;Val:Integer):	Integer;
	begin
		Result:=Val;
		if	Min<Max	then
		begin
			if	Val<Min	then	Result:=Min
			else
			if	Val>Max	then	Result:=Max;
		end;
	end;

var
	R:	TRect;
begin
	if	AllowFollowing	then
	begin
(*
		if	UnionAsocWindows	and
				(not	IsModalWindow(WindowParams.Handle))	and
				IsCondOk(WindowParams,cfAsocWindows)	then
			R:=GetAsocWindowsRect(WindowParams.Handle)
		else
		begin

			if	not	IsWindowStable(WindowParams.Handle)	then	Exit;
			R:=WindowParams.WindowRect;
		end;
*)

		R:=WindowParams.WindowRect;

		if	IsRectEmpty(R)	then	Exit;

//		Pt:=R.TopLeft;
		pt:=GetAnchorPoint(R,WindowAnchor);
		OffsetPt(pt,GetAnchorPoint(MascotRect,MascotAnchor),True);
		OffsetPt(pt,XPos,YPos);
	end;

	if	AlwaysInScreen	then
	begin
		with	MascotRect,pt	do
		begin
			x:=AdjustValue(-Left,Screen.Width-Right,x);
			y:=AdjustValue(-Top,Screen.Height-Bottom,y);
		end;
	end;
end;

//------------------------------------------------------------

procedure	TDocForm1.InitFolderMenuList;
begin
{
	with	FolderMenuList	do
	begin
		//üե
		Add('DESKTOP'										,CSIDL_DESKTOP									,True	);
		Add('DRIVES'										,CSIDL_DRIVES										,True	);
		Add('NETWORK'										,CSIDL_NETWORK									,True	);
		Add('CONTROLS'									,CSIDL_CONTROLS									,True	);
		Add('PRINTERS'									,CSIDL_PRINTERS									,False);
		Add('BITBUCKET'									,CSIDL_BITBUCKET								,False);

		//ƥե
		Add('STARTMENU'									,CSIDL_STARTMENU								,True	);
		Add('COMMON_STARTMENU'					,CSIDL_COMMON_STARTMENU					,True	);
		Add('PROGRAMS'									,CSIDL_PROGRAMS									,True	);
		Add('PROGRAM_FILES'							,CSIDL_PROGRAM_FILES						,True	);
		Add('PROGRAM_FILESX86'					,CSIDL_PROGRAM_FILESX86					,True	);
		Add('COMMON_PROGRAMS'						,CSIDL_COMMON_PROGRAMS					,True	);
		Add('PROGRAM_FILES_COMMON'			,CSIDL_PROGRAM_FILES_COMMON			,True	);
		Add('PROGRAM_FILES_COMMONX86'		,CSIDL_PROGRAM_FILES_COMMONX86	,True	);
		Add('STARTUP'										,CSIDL_STARTUP									,True	);
		Add('COMMON_STARTUP'						,CSIDL_COMMON_STARTUP						,True	);
		Add('ALTSTARTUP'								,CSIDL_ALTSTARTUP								,True	);
		Add('COMMON_ALTSTARTUP'					,CSIDL_COMMON_ALTSTARTUP				,True	);
		Add('DESKTOPDIRECTORY'					,CSIDL_DESKTOPDIRECTORY					,True	);
		Add('COMMON_DESKTOPDIRECTORY'		,CSIDL_COMMON_DESKTOPDIRECTORY	,True	);

		//Windowsե
		Add('WINDOWS'										,CSIDL_WINDOWS									,True	);
		Add('SYSTEM'										,CSIDL_SYSTEM										,True	);
		Add('SYSTEMX86'									,CSIDL_SYSTEMX86								,True	);
		Add('PROFILE'										,CSIDL_PROFILE									,True	);
		Add('SENDTO'										,CSIDL_SENDTO										,True	);
		Add('FONTS'											,CSIDL_FONTS										,True	);
		Add('CONNECTIONS'								,CSIDL_CONNECTIONS							,True	);
		Add('RECENT'										,CSIDL_RECENT										,False);
		Add('NETHOOD'										,CSIDL_NETHOOD									,False);
		Add('PRINTHOOD'									,CSIDL_PRINTHOOD								,False);

		//Internet
		Add('INTERNET'									,CSIDL_INTERNET									,False);
		Add('INTERNET_CACHE'						,CSIDL_INTERNET_CACHE						,False);
		Add('COOKIES'										,CSIDL_COOKIES									,False);
		Add('HISTORY'										,CSIDL_HISTORY									,False);
		Add('FAVORITES'									,CSIDL_FAVORITES								,True	);
		Add('COMMON_FAVORITES'					,CSIDL_COMMON_FAVORITES					,True	);

		//Personal
		Add('PERSONAL'									,CSIDL_PERSONAL									,True	);
		Add('COMMON_DOCUMENTS'					,CSIDL_COMMON_DOCUMENTS					,True	);
		Add('MYPICTURES'								,CSIDL_MYPICTURES								,True	);
		Add('APPDATA'										,CSIDL_APPDATA									,True	);
		Add('COMMON_APPDATA'						,CSIDL_COMMON_APPDATA						,True	);
		Add('LOCAL_APPDATA'							,CSIDL_LOCAL_APPDATA						,True	);
		Add('TEMPLATES'									,CSIDL_TEMPLATES								,False);
		Add('COMMON_TEMPLATES'					,CSIDL_COMMON_TEMPLATES					,False);
		Add('ADMINTOOLS'								,CSIDL_ADMINTOOLS								,False);
		Add('COMMON_ADMINTOOLS'					,CSIDL_COMMON_ADMINTOOLS				,False);
	end;
}
end;

procedure	TDocForm1.RandomChange(IgnoreCurrent:Boolean);
var
	i:	Integer;
	Total:	Integer;
	Index:	Integer;
	rnd:	Real;
begin
	//иΨû
	Total:=0;
	for	i:=0	to	MascotList.Count-1	do
	begin
		if	IgnoreCurrent	and	(i=CurrentMascot)	then	Continue;
		Inc(Total,MascotList[i].Frequency);
	end;

	//ڤؤоݤʤ
	if	Total=0	then	Exit;

	rnd:=Random(Total);

	Index:=0;
	for	i:=0	to	MascotList.Count-1	do
	begin
		if	IgnoreCurrent	and	(i=CurrentMascot)	then	Continue;
		Index:=i;
		rnd:=rnd-MascotList[i].Frequency;
		if	rnd<0	then	Break;
	end;

	CurrentMascot:=Index;
end;

{
var
	Count:	Integer;
	Index:	Integer;
begin
	Count:=MascotList.Count;
	if	IgnoreCurrent	then	Dec(Count);
	Index:=Round(Random(Count));
	if	IgnoreCurrent	and	(Index=CurrentMascot)	then	Inc(Index);
	CurrentMascot:=Index;
end;
}

procedure	TDocForm1.SetProperty(var	Prop:Boolean;NewVal:Boolean);
begin
	if	Prop<>NewVal	then
	begin
		Prop:=NewVal;
		Modified:=True;
	end;
end;

procedure	TDocForm1.SetProperty(var	Prop:Integer;NewVal:Integer);
begin
	if	Prop<>NewVal	then
	begin
		Prop:=NewVal;
		Modified:=True;
	end;
end;

procedure	TDocForm1.SetProperty(var	Prop:string;NewVal:string);
begin
	if	Prop<>NewVal	then
	begin
		Prop:=NewVal;
		Modified:=True;
	end;
end;

//------------------------------------------------------------

{	Access Method	}

function	TDocForm1.GetTopMost:	Boolean;
begin
	Result:=Field.TopMost;
end;

procedure	TDocForm1.SetTopMost(Val:Boolean);
begin
	SetProperty(FField.TopMost,Val);
end;

function	TDocForm1.GetAlwaysInScreen:	Boolean;
begin
	Result:=Field.AlwaysInScreen;
end;

procedure	TDocForm1.SetAlwaysInScreen(Val:Boolean);
begin
	SetProperty(FField.AlwaysInScreen,Val);
end;

function	TDocForm1.GetZOrderControl:	Boolean;
begin
	Result:=Field.ZOrderControl;
end;

procedure	TDocForm1.SetZOrderControl(Val:Boolean);
begin
	SetProperty(FField.ZOrderControl,Val);
end;

function	TDocForm1.GetTimerInterval:	Integer;
begin
	Result:=Field.TimerInterval;
end;

procedure	TDocForm1.SetTimerInterval(Val:Integer);
begin
	SetProperty(FField.TimerInterval,Val);
end;

function	TDocForm1.GetChangeInterval:	Integer;
begin
	Result:=Field.ChangeInterval;
end;

procedure	TDocForm1.SetChangeInterval(Val:Integer);
begin
	SetProperty(FField.ChangeInterval,Val);
end;

function	TDocForm1.GetInitialChange:	Boolean;
begin
	Result:=Field.InitialChange;
end;

procedure	TDocForm1.SetInitialChange(Val:Boolean);
begin
	SetProperty(FField.InitialChange,Val);
end;

function	TDocForm1.GetUseCaptionFmt:	Boolean;
begin
	Result:=Field.UseCaptionFmt;
end;

procedure	TDocForm1.SetUseCaptionFmt(Val:Boolean);
begin
	SetProperty(FField.UseCaptionFmt,Val);
end;

function	TDocForm1.GetCaptionFmt:	string;
begin
	Result:=Field.CaptionFmt;
end;

procedure	TDocForm1.SetCaptionFmt(Val:string);
begin
	SetProperty(FField.CaptionFmt,Val);
end;

function	TDocForm1.GetDefButtonEnable:	Boolean;
begin
	Result:=Field.DefButtonEnable;
end;

procedure	TDocForm1.SetDefButtonEnable(Val:Boolean);
begin
	SetProperty(FField.DefButtonEnable,Val);
end;

function	TDocForm1.GetSmartMove:	Boolean;
begin
	Result:=Field.SmartMove;
end;

procedure	TDocForm1.SetSmartMove(Val:Boolean);
begin
	SetProperty(FField.SmartMove,Val);
end;

function	TDocForm1.GetUsePlugin:	Boolean;
begin
	Result:=Field.UsePlugin;
end;

procedure	TDocForm1.SetUsePlugin(Val:Boolean);
begin
	SetProperty(FField.UsePlugin,Val);
end;

function	TDocForm1.GetDisableFollowings:	TDisableFollowings;
begin
	Result:=Field.DisableFollowings;
end;

procedure	TDocForm1.SetDisableFollowings(Val:TDisableFollowings);
begin
	if	DisableFollowings<>Val	then
	begin
		FField.DisableFollowings:=Val;
		Modified:=True;
	end;
end;

function	TDocForm1.GetAllowDuplication:	Boolean;
begin
	Result:=Field.AllowDuplication;
end;

procedure	TDocForm1.SetAllowDuplication(Val:Boolean);
begin
	SetProperty(FField.AllowDuplication,Val);
end;

function	TDocForm1.GetShowHint:	Boolean;
begin
	Result:=Field.ShowHint;
end;

procedure	TDocForm1.SetShowHint(Val:Boolean);
begin
	SetProperty(FField.ShowHint,Val);
end;

{
function	TDocForm1.GetEnablePluginDirs:	TEnablePluginDirs;
begin
	Result:=Field.EnablePluginDirs;
end;

procedure	TDocForm1.SetEnablePluginDirs(Val:TEnablePluginDirs);
begin
	if	EnablePluginDirs<>Val	then
	begin
		FField.EnablePluginDirs:=Val;
		Modified:=True;
	end;
end;
}

function	TDocForm1.GetHintColor:	TColor;
begin
	Result:=Field.HintColor;
end;

procedure	TDocForm1.SetHintColor(Val:TColor);
begin
	if	HintColor<>Val	then
	begin
		FField.HintColor:=Val;
		Modified:=True;
	end;
end;

function	TDocForm1.GetUnionAsocWindows:	Boolean;
begin
	Result:=Field.UnionAsocWindows;
end;

procedure	TDocForm1.SetUnionAsocWindows(Val:Boolean);
begin
	SetProperty(FField.UnionAsocWindows,Val);
end;

function	TDocForm1.GetHideByFullScreenWindow:	Boolean;
begin
	Result:=Field.HideByFullScreenWindow;
end;

procedure	TDocForm1.SetHideByFullScreenWindow(Val:Boolean);
begin
	SetProperty(FField.HideByFullScreenWindow,Val);
end;

function	TDocForm1.GetEnableActivateModalDialog:	Boolean;
begin
	Result:=Field.EnableActivateModalDialog;
end;

procedure	TDocForm1.SetEnableActivateModalDialog(Val:Boolean);
begin
	SetProperty(FField.EnableActivateModalDialog,Val);
end;

function	TDocForm1.GetSensorIDE:	Boolean;
begin
	Result:=Field.SensorIDE;
end;

procedure	TDocForm1.SetSensorIDE(Val:Boolean);
begin
	SetProperty(FField.SensorIDE,Val);
end;

function	TDocForm1.GetPluginDirs:	string;
begin
	Result:=Field.PluginDirs;
end;

procedure	TDocForm1.SetPluginDirs(Val:string);
begin
	if	PluginDirs<>Val	then
	begin
		FField.PluginDirs:=Val;
		Modified:=True;
	end;
end;

{
function	TDocForm1.GetBlinkerParams:	TAlarmParams;
begin
	Result:=Field.BlinkerParams;
end;

procedure	TDocForm1.SetBlinkerParams(Val:TAlarmParams);
begin
	FField.BlinkerParams:=Val;
	Modified:=True;
end;

function	TDocForm1.GetShakerParams:	TAlarmParams;
begin
	Result:=Field.ShakerParams;
end;

procedure	TDocForm1.SetShakerParams(Val:TAlarmParams);
begin
	FField.ShakerParams:=Val;
	Modified:=True;
end;

function	TDocForm1.GetPlayerParams:	TAlarmParams;
begin
	Result:=Field.PlayerParams;
end;

procedure	TDocForm1.SetPlayerParams(Val:TAlarmParams);
begin
	FField.PlayerParams:=Val;
	Modified:=True;
end;
}

function	TDocForm1.GetTimeOffset:	Integer;
begin
	Result:=Field.TimeOffset;
end;

procedure	TDocFOrm1.SetTimeOffset(Val:Integer);
begin
	SetProperty(FField.TimeOffset,Val);
end;

function	TDocForm1.GetUseDesktop:	Boolean;
begin
	Result:=Field.UseDesktop;
end;

procedure	TDocFOrm1.SetUseDesktop(Val:Boolean);
begin
	SetProperty(FField.UseDesktop,Val);
end;

function	TDocForm1.GetIconState:	TIconState;
begin
	Result:=Field.IconState;
end;

procedure	TDocForm1.SetIconState(Val:TIconState);
begin
	if	IconState<>Val	then
	begin
		FField.IconState:=Val;
		Modified:=True;
	end;
end;

function	TDocForm1.GetNTPServer:	string;
begin
	Result:=Field.NTPServer;
end;

procedure	TDocForm1.SetNTPServer(Val:string);
begin
	SetProperty(FField.NTPServer,Val);
end;

function	TDocForm1.GetNTPTimeout:	Integer;
begin
	Result:=Field.NTPTimeout;
end;

procedure	TDocForm1.SetNTPTimeout(Val:Integer);
begin
	SetProperty(FField.NTPTimeout,Val);
end;

function	TDocForm1.GetNTPInterval:	TNTPInterval;
begin
	Result:=Field.NTPInterval;
end;

procedure	TDocForm1.SetNTPInterval(Val:TNTPInterval);
begin
	if	NTPInterval<>Val	then
	begin
		FField.NTPInterval:=Val;
		Modified:=True;
	end;
end;

function	TDocForm1.GetNTPLastUpdate:	TDateTime;
begin
	Result:=Field.NTPLastUpdate;
end;

procedure	TDocForm1.SetNTPLastUpdate(Val:TDateTime);
begin
	if	NTPLastUpdate<>Val	then
	begin
		FField.NTPLastUpdate:=Val;
		Modified:=True;
	end;
end;

function	TDocForm1.GetNTPSyncPeriodic:	Boolean;
begin
	Result:=Field.NTPSyncPeriodic;
end;

procedure	TDocForm1.SetNTPSyncPeriodic(Val:Boolean);
begin
	SetProperty(FField.NTPSyncPeriodic,Val);
end;

function	TDocForm1.GetNTPSyncSystem:	Boolean;
begin
	Result:=Field.NTPSyncSystem;
end;

procedure	TDocForm1.SetNTPSyncSystem(Val:Boolean);
begin
	SetProperty(FField.NTPSyncSystem,Val);
end;

function	TDocForm1.GetNTPSyncGradually:	Boolean;
begin
	Result:=Field.NTPSyncGradually;
end;

procedure	TDocForm1.SetNTPSyncGradually(Val:Boolean);
begin
	SetProperty(FField.NTPSyncGradually,Val);
end;

function	TDocForm1.GetNTPOnlyIfConnected:	Boolean;
begin
	Result:=Field.NTPOnlyIfConnected;
end;

procedure	TDocForm1.SetNTPOnlyIfConnected(Val:Boolean);
begin
	SetProperty(FField.NTPOnlyIfConnected,Val);
end;

function	TDocForm1.GetNTPServerValid:	Boolean;
begin
	Result:=Field.NTPServerValid;
end;

procedure	TDocForm1.SetNTPServerValid(Val:Boolean);
begin
	SetProperty(FField.NTPServerValid,Val);
end;

//--

function	TDocForm1.GetCurrentMascot:	Integer;
begin
	Result:=MascotList.ItemIndex;
end;

procedure	TDocForm1.SetCurrentMascot(Index:Integer);
begin
	with	MascotList	do
		if	ItemIndex<>Index	then
		begin
			ItemIndex:=Index;
			if	Index>=0	then
				WindowFollowing:=Items[ItemIndex].WindowFollowing;
			Modified:=True;
		end;
end;

procedure	TDocForm1.SetWindowFollowing(Val:Boolean);
begin
	FWindowFollowing:=Val;
	if	MascotList.Selected	then
		MascotList[MascotList.ItemIndex].WindowFollowing:=Val;
end;

function	TDocForm1.GetMascotAnchor:	TAnchor;
begin
	with	MascotList	do
		if	ItemIndex<0	then
			Result:=BottomLeft
		else
			Result:=Items[ItemIndex].MascotAnchor;
end;

function	TDocForm1.GetWindowAnchor:	TAnchor;
begin
	with	MascotList	do
		if	ItemIndex<0	then
			Result:=BottomRight
		else
			Result:=Items[ItemIndex].WindowAnchor;
end;

function	TDocForm1.GetXPos:	Integer;
begin
	with	MascotList	do
		if	ItemIndex<0	then
			Result:=0
		else
			Result:=Items[ItemIndex].XPos;
end;

function	TDocForm1.GetYPos:	Integer;
begin
	with	MascotList	do
		if	ItemIndex<0	then
			Result:=0
		else
			Result:=Items[ItemIndex].YPos;
end;

function	TDocForm1.GetModifiedNow:	TDateTime;
begin
	Result:=Now+TimeDifference/(60*60*24);
end;

function	TDocForm1.GetModifiedDate:	TDateTime;
begin
	Result:=Trunc(ModifiedNow);
end;

function	TDocForm1.GetModifiedTime:	TDateTime;
begin
	Result:=Frac(ModifiedNow);
end;

//------------------------------------------------------------

{
//ץեȤ¸
procedure	TDocForm1.StoreACFont;
var
	NCMetrics:	TNonClientMetrics;
begin
	NCMetrics.cbSize:=SizeOf(NCMetrics);
	SystemParametersInfo(SPI_GETNONCLIENTMETRICS,SizeOf(NCMetrics),@NCMetrics,0);
	OrgACFont:=NCMetrics.lfCaptionFont;
	OrgACColor:=GetSysColor(COLOR_CAPTIONTEXT);
	ACFontModified:=True;
end;

//¸ץեȤ򸵤᤹
procedure	TDocForm1.RestoreACFont;
var
	NCMetrics:	TNonClientMetrics;
	nDspElements:	Integer;
begin
	if	ACFontModified	then
	begin
		NCMetrics.cbSize:=SizeOf(NCMetrics);
		SystemParametersInfo(SPI_GETNONCLIENTMETRICS,SizeOf(NCMetrics),@NCMetrics,0);
		NCMetrics.lfCaptionFont:=OrgACFont;
		SystemParametersInfo(SPI_SETNONCLIENTMETRICS,SizeOf(NCMetrics),@NCMetrics,0);
		nDspElements:=COLOR_CAPTIONTEXT;
		SetSysColors(1,nDspElements,OrgACColor);
		ACFontModified:=False;
	end;
end;

//ץեȤȿ
procedure	TDocForm1.UpdateACFont;
var
	NCMetrics:	TNonClientMetrics;
	nDspElements:	Integer;
begin
	with	NCMetrics	do
	begin
		cbSize:=SizeOf(NCMetrics);
		SystemParametersInfo(SPI_GETNONCLIENTMETRICS,SizeOf(NCMetrics),@NCMetrics,0);

		if	CompareMem(@ACFont,@lfCaptionFont,SizeOf(TLogFont))=False	then
		begin
			nDspElements:=COLOR_CAPTIONTEXT;
			if	ACFontEnable	then
			begin
				//ȼ˥ץΥեȤ
				lfCaptionFont:=ACFont;
				SetSysColors(1,nDspElements,ACColor);
			end	else	begin
				//᤹
				lfCaptionFont:=OrgACFont;
				SetSysColors(1,nDspElements,OrgACColor);
			end;

			SystemParametersInfo(SPI_SETNONCLIENTMETRICS,SizeOf(NCMetrics),@NCMetrics,0);
			ACFontModified:=ACFontEnable;
		end;
	end;
end;
}

{
initialization
	QrekConverter:=TQrekConverter.Create;
finalization
	QrekConverter.Free;
}

end.

