unit Mascot;

interface

uses
	SysUtils, Types, Classes, Variants, QTypes, QGraphics, QControls, QForms,
	QDialogs, QStdCtrls, QImgList, QExtCtrls, QMenus, contnrs, MakeMask;

(*
	Windows, SysUtils, Classes, Forms, Graphics, contnrs, MakeMask;
*)

type
	TAnchor=(TopLeft,TopRight,BottomLeft,BottomRight);
	TMaskType=(maskNormal,mask4,mask8,maskAux);

type
	TMascot=class(TObject)
	private
		FName:	string;
		FPicture:	string;
		FMask:	string;
		FMaskType:	TMaskType;
		FMascotAnchor:	TAnchor;
		FWindowAnchor:	TAnchor;
		FWindowFollowing:	Boolean;	//*
		FXPos:	Integer;
		FYPos:	Integer;
		FFrequency:	Integer;
	protected
		procedure	SetPicture(APicture:string);
	public
		constructor	Create(AName,APicture:string);	overload;
		constructor	Create(APicture:string);	overload;
		procedure	Assign(AMascot:TMascot);
		property	Name:	string	read	FName	write	FName;
		property	Picture:	string	read	FPicture	write	SetPicture;
		property	Mask:	string	read	FMask	write	FMask;
		property	MaskType:	TMaskType	read	FMaskType	write	FMaskType;
		property	MascotAnchor:	TAnchor	read	FMascotAnchor	write	FMascotAnchor;
		property	WindowAnchor:	TAnchor	read	FWindowAnchor	write	FWindowAnchor;
		property	WindowFollowing:	Boolean	read	FWindowFollowing	write	FWindowFollowing;
		property	XPos:	Integer	read	FXPos	write	FXPos;
		property	YPos:	Integer	read	FYPos	write	FYPos;
		property	Frequency:	Integer	read	FFrequency	write	FFrequency;
	end;
	PMascot=^TMascot;

	TMascotList=class(TObjectList)
	private
		FModified:	Boolean;
		FPicture:	TPicture;
		FMask:	TBitmap;
		FHasPicture:	Boolean;
		FItemIndex:	Integer;
		FOnChange:	TNotifyEvent;
	protected
		function	GetItem(Idx:Integer):	TMascot;
		procedure	SetItem(Idx:Integer;AMascot:TMascot);
		function	GetCurrentMascot:	TMascot;
		function	GetPicture:	TPicture;
		function	GetMask:	TBitmap;
		function	GetSelected:	Boolean;
		procedure	SetModified(Val:Boolean);
	public
		constructor	Create;
		destructor	Destroy;	override;
		procedure	Clear;	override;
		function	Add(AMascot:TMascot):	Integer;
		procedure	Assign(AMascotList:TMascotList);
		procedure	Delete(Index:Integer);
		function	Exchange(Index:Integer):	Boolean;
		procedure	SetItemIndex(Index:Integer);
		procedure	LoadPicture;
		property	Items[Idx:Integer]:	TMascot	read	GetItem	write	SetItem;	default;
		property	Modified:	Boolean	read	FModified	write	SetModified;
		property	ItemIndex:	Integer	read	FItemIndex	write	SetItemIndex;
		property	CurrentMascot:	TMascot	read	GetCurrentMascot;
		property	Picture:	TPicture	read	GetPicture;
		property	Mask:	TBitmap	read	GetMask;
		property	HasPicture:	Boolean	read	FHasPicture;
		property	Selected:	Boolean	read	GetSelected;
		property	OnChange:	TNotifyEvent	read	FOnChange	write	FOnChange;
	end;

implementation

//------------------------------------------------------------
//  TMascot
//------------------------------------------------------------

constructor	TMascot.Create(APicture:string);
begin
	Picture:=APicture;
	FMascotAnchor:=BottomLeft;
	FWindowAnchor:=BottomRight;
	FWindowFollowing:=True;
end;

constructor	TMascot.Create(AName,APicture:string);
begin
	FName:=AName;
	Create(APicture);
end;

procedure	TMascot.SetPicture(APicture:string);
begin
	FPicture:=APicture;
	if	FName=''	then
		FName:=ChangeFileExt(ExtractFileName(FPicture),'');
end;

procedure	TMascot.Assign(AMascot:TMascot);
begin
	Name								:=AMascot.Name;
	Picture							:=AMascot.Picture;
	Mask								:=AMascot.Mask;
	MaskType						:=AMascot.MaskType;
	MascotAnchor				:=AMascot.MascotAnchor;
	WindowAnchor				:=AMascot.WindowAnchor;
	WindowFollowing			:=AMascot.WindowFollowing;
	XPos								:=AMascot.XPos;
	YPos								:=AMascot.YPos;
	Frequency						:=AMascot.Frequency;
end;

//------------------------------------------------------------
//  TMascotList
//------------------------------------------------------------

constructor	TMascotList.Create;
begin
	inherited	Create(True);

	FPicture:=TPicture.Create;
	FMask:=TBitmap.Create;
	FModified:=True;
	ItemIndex:=-1;
end;

destructor	TMascotList.Destroy;
begin
	FPicture.Free;
	FMask.Free;

	inherited;
end;

procedure	TMascotList.Clear;
begin
	inherited;

	ItemIndex:=-1;
end;

function	TMascotList.Add(AMascot:TMascot):	Integer;
begin
	Modified:=True;
	Result:=inherited	Add(AMascot);
end;

procedure	TMascotList.Delete(Index:Integer);
begin
	inherited	Delete(Index);

	Modified:=True;
	if	ItemIndex>Index	then
		ItemIndex:=ItemIndex-1;
	if	ItemIndex>Count-1	then
		ItemIndex:=Count-1;
end;

function	TMascotList.GetItem(Idx:Integer):	TMascot;
begin
	Result:=TObject(List[Idx])	as	TMascot;
end;

procedure	TMascotList.SetItem(Idx:Integer;AMascot:TMascot);
begin
	List[Idx]:=AMascot;
	Modified:=True;
end;

procedure	TMascotList.Assign(AMascotList:TMascotList);
var
	i:	Integer;
	AMascot:	TMascot;
begin
	Clear;
	for	i:=0	to	AMascotList.Count-1	do
	begin
		AMascot:=TMascot.Create;
		AMascot.Assign(AMascotList[i]);
		Add(AMascot);
	end;
	ItemIndex:=AMascotList.ItemIndex;
	Modified:=True;
end;

//Itemȸ򴹤
function	TMascotList.Exchange(Index:Integer):	Boolean;
var
	AMascot:	TMascot;
begin
	Result:=(Index>0)	and	(Index<Count);

	if	Result	then
	begin
		AMascot:=Items[Index];
		Items[Index]:=Items[Index-1];
		Items[Index-1]:=AMascot;

		if	ItemIndex=Index	then
			ItemIndex:=Index-1
		else
		if	ItemIndex=Index-1	then
			ItemIndex:=Index;

		Modified:=True;
	end;
end;

procedure	TMascotList.SetItemIndex(Index:Integer);
begin
	if	(Index>=0)	and	(Index<Count)	then
	begin
		if	FItemIndex<>Index	then
		begin
			FItemIndex:=Index;
			Modified:=True;
//			LoadPicture;
		end;
	end	else	begin
		FHasPicture:=False;
		FItemIndex:=-1;
	end;
end;

function	TMascotList.GetCurrentMascot:	TMascot;
begin
	if	Count>0	then
		Result:=Items[ItemIndex]
	else
		Result:=nil;
end;

function	TMascotList.GetSelected:	Boolean;
begin
	Result:=ItemIndex>=0;
end;

procedure	TMascotList.SetModified(Val:Boolean);
begin
	if	Assigned(FOnChange)	then
		OnChange(Self);
	FModified:=Val;
end;

procedure	TMascotList.LoadPicture;
begin
	if	ItemIndex>=0	then
	begin
		try
			FPicture.LoadFromFile(Items[ItemIndex].Picture);
		except
			on	Exception	do
			begin
				FHasPicture:=False;
				FModified:=False;
				raise;
			end;
{
			on	EInvalidGraphic	do
			begin
//				Application.MessageBox('ͭʲեǤϤޤ','ActMascot',MB_OK	or	MB_ICONWARNING);
				FHasPicture:=False;
				FModified:=False;
//				FItemIndex:=-1;
				raise;
			end;
			on	EFOpenError	do
			begin
//				Application.MessageBox('ե뤬ޤ','ActMascot',MB_OK	or	MB_ICONWARNING);
				FHasPicture:=False;
				FModified:=False;
//				FItemIndex:=-1;
				raise;
			end;
}
		end;
		FHasPicture:=True;
		FModified:=False;
		case	Items[ItemIndex].FMaskType	of
			maskNormal:
				GraphicToMask(FPicture.Graphic,FMask);
			mask4:
				MakeMaskDef(FPicture.Graphic,FMask,False);
			mask8:
				MakeMaskDef(FPicture.Graphic,FMask,True);
			maskAux:
			begin
				FMask.LoadFromFile(Items[ItemIndex].Mask);
				FMask.Mask(FMask.Canvas.Pixels[0,0]);
			end;
//			FMask.PixelFormat:=pf8bit;
		end;
	end;
end;

function	TMascotList.GetPicture:	TPicture;
begin
	if	Modified	then
		LoadPicture;

	if	HasPicture	then
		Result:=FPicture
	else
		Result:=nil;
end;

function	TMascotList.GetMask:	TBitmap;
begin
	if	Modified	then
		LoadPicture;

	if	HasPicture	then
		Result:=FMask
	else
		Result:=nil;
end;

end.

